/*
 * PSwing Utilities -- Nifty Swing Widgets
 * Copyright (C) 2002  Pallas Technology
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Pallas Technology
 * 1170 HOWELL MILL RD NW
 * SUITE 306
 * ATLANTA GEORGIA 30318
 * 
 * PHONE 404.983.0623
 * EMAIL info@pallastechnology.com
 * 
 * www.pallastechnology.com
 **************************************************************************
 * $Archive: SwingTools$
 * $FileName: DateComboBox.java$
 * $FileID: 8$
 *
 * Last change:
 * $AuthorName: Klaue$
 * $Date: 02.01.07 04:47$
 * $VerID: ?$
 * $Comment: Update to Java 1.5, Minor makeup, including the class DateFocusListener (also
 *           from Pallas Technology) to this file $
 **************************************************************************/
//////////////////////////////////////////////////////////////
// DateComboBox.java
//////////////////////////////////////////////////////////////

import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.ComboBoxUI;
import javax.swing.plaf.basic.ComboPopup;
import javax.swing.plaf.metal.MetalComboBoxUI;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;
import javax.swing.border.EmptyBorder;

import Helpers.Dictionary;

//////////////////////////////////////////////////////////////

/**
 * Title:   $FileName: DateComboBox.java$
 * @version $VerNum: 10$
 * @author $AuthorName: Rob MacGrogan$<br><br>
 * 
 * $Description: A date control that pops up a calendar.$<br>
 * $KeyWordsOff: $<br><br>
 * 
 * 
 * A date control that pops up a calendar. Derived from code posted at
 * http://softwaredev.earthweb.com/java/article/0,,12082_735291,00.html by
 * Paul Book.<br><br>
 * 
 * Works under java 1.3 and 1.4.1. A bug in 1.4.0 causes very odd behavior
 * under Windows LAF, but should work fine under other LAFs.
 */
@SuppressWarnings("serial")
public class DateComboBox extends JComboBox {

  //Used only for display.
  protected SimpleDateFormat dateFormat = new SimpleDateFormat("MMM d, yyyy");

  //Used for formatting what user types.
  private ArrayList<DateFormat> formats = new ArrayList<DateFormat>();

  /**
   * Defaults to null. And MMM d, yyyy pattern.
   */
  public DateComboBox() {
    super();
    initializeComponent();
  }

  /**
   * Sets initial date of combo box. Defaults pattern to MMM d, yyyy.
   */
  public DateComboBox(java.util.Date date) {
    super();
    initializeComponent();
    setDate(date);
  }

  public DateComboBox(java.util.Date date, String pattern) {
    super();
    initializeComponent();
    setDateFormat(pattern);
    setDate(date);
  }

  public DateComboBox(String pattern) {
    super();
    initializeComponent();
    setDateFormat(pattern);
  }

  @Override
public void setName(String s) {
    super.setName(s);
    getEditor().getEditorComponent().setName(s + "-- editor");
  }

  DateFormat getFormat() {
    return this.dateFormat;
  }

  public void addFormat(DateFormat format){
    this.formats.add(format);
  }

  public void addFormat(String format){
    this.formats.add(new SimpleDateFormat(format));
  }

  /**
   * Initializes the DateComboBox.
   */
  private void initializeComponent() {
    //super.setEditable(true);

    JTextField field = (JTextField) getEditor().getEditorComponent();
    field.addFocusListener(new DateFocusListener(this));
    //setCalendarIcon();
  }
  
  /**
   * Attempts to parse sDate to a date using all available date formats.
   */
  java.util.Date parseDateString(String sDate)
          throws ParseException{
    java.util.Date value = null;
    //Now try all remaining formats.
    Iterator<DateFormat> itr = this.formats.iterator();
    while(itr.hasNext()){
      DateFormat format = itr.next();
      try{
        value = format.parse(sDate);
        //if we get here we got a value, so break.
        break;
      }
      catch (ParseException ex2){
        //just don't break.
      }
    }
    if (value == null){
      throw new ParseException("Can't format string " + sDate + " into a date.", 0);
    }
    return value;
  }

 /* private void setCalendarIcon() {
    Component[] components = getComponents();
    for (int i = 0; i < components.length; i++) {
      if (components[i] instanceof JButton) {
        JButton btn = (JButton) components[i];
        ActionListener[] listeners = btn.getActionListeners();
        JButton calButton = new JButton(getButtonIcon());
        for (int j = 0; j < listeners.length; j++) {
          calButton.addActionListener(listeners[i]);
        }
        remove(btn);
        add(calButton);
        //btn.setIcon(getButtonIcon());
        break;
      }
    }

  }*/

  /*private Icon getButtonIcon() {
    java.net.URL imageURL = DateComboBox.class.getResource("calendar.gif");
    Image img = Toolkit.getDefaultToolkit().getImage(imageURL);
    Icon ic = new ImageIcon(img);
    return ic;
  }*/

  /**
   * This method does nothing. DateComboBox is always editable.
   */
  @Override
public void setEditable(boolean editable) {
    //Do nothing.
  }

  public void setDateFormat(SimpleDateFormat dateFormat) {
    this.dateFormat = dateFormat;
  }

  public void setDateFormat(String pattern) {
    this.dateFormat = new SimpleDateFormat(pattern);
  }

  /**
   * Returns a new Date with the same calendar date as the Date passed in, 
   * but with the time changed to 12:00 AM.
   */
  public static java.util.Date stripTime(java.util.Date dt){
    Calendar cal = Calendar.getInstance();
    cal.setTime(dt);
    int year = cal.get(Calendar.YEAR);
    int month = cal.get(Calendar.MONTH);
    int day = cal.get(Calendar.DAY_OF_MONTH);
    
    Calendar calStripped = Calendar.getInstance();
    calStripped.clear();
    calStripped.set(year, month, day);
    
    return calStripped.getTime(); 
  }


  public void setDate(java.util.Date date) {
    if (date != null) {
      date = stripTime(date);
      CBDate cbDate = new CBDate();
      cbDate.setDate(date);
      cbDate.setFormat(this.dateFormat.toPattern());
      setSelectedItem(cbDate);
    }
    else {
      setSelectedItem(null);
    }
  }

  @SuppressWarnings("null")
public java.util.Date getDate() {
    java.util.Date date = null;
    try {
      Object o = getSelectedItem();
      if (o instanceof CBDate) {
        CBDate cbDate = (CBDate) o;
        date = cbDate.getDate();
      }
      else {
        throw new java.text.ParseException("The value of the DateComboBox is not a Date.", 0);
      }
    }
    catch (ParseException ex) {
      date = null;
    }
    return date;
  }

  @Override
public void setSelectedItem(Object item) {
    // Could put extra logic here or in renderer when item is instanceof Date, Calendar, or String
    // Dont keep a list ... just the currently selected item
    removeAllItems(); // hides the popup if visible
    addItem(item);
    super.setSelectedItem(item);
  }

  @Override
public void updateUI() {
	  // ****************************************************************
	  // edited by klaue to use the metalsatecombobox for every
	  // LAF that is not extra handled. i also added the LAFS i use in GalleryGrabber
    ComboBoxUI cui = new MetalDateComboBoxUI();
    setUI(cui);
    // super.updateUI();
    initializeComponent();
  }

  // Inner classes are used purely to keep DateComboBox component in one file
  //////////////////////////////////////////////////////////////
  // UI Inner classes -- one for each supported Look and Feel
  //////////////////////////////////////////////////////////////

  class MetalDateComboBoxUI extends MetalComboBoxUI {
    @Override
	protected ComboPopup createPopup() {
      return new DatePopup((DateComboBox) this.comboBox);
    }
  }

  class CBDate {
    private java.util.Date date = null;
    private SimpleDateFormat format = null;

    public java.util.Date getDate() {
      return this.date;
    }
    public void setDate(java.util.Date dt) {
      this.date = dt;
    }
    public void setFormat(String s) {
      this.format = new SimpleDateFormat(s);
    }

    @Override
	public String toString() {
      return this.format.format(this.date);
    }
  }

  //////////////////////////////////////////////////////////////
  // DatePopup inner class
  //////////////////////////////////////////////////////////////

  class DatePopup implements ComboPopup, MouseMotionListener, MouseListener, KeyListener, PopupMenuListener {

    protected DateComboBox comboBox;
    protected Calendar calendar;
    protected JPopupMenu popup;
    protected JLabel monthLabel;
    protected JPanel days = null;
    protected SimpleDateFormat monthFormat = new SimpleDateFormat("MMM yyyy");

    protected Color selectedBackground;
    protected Color selectedForeground;
    protected Color background;
    protected Color foreground;

    public DatePopup(DateComboBox comboBox) {
      this.comboBox = comboBox;
      this.calendar = Calendar.getInstance();
      // check Look and Feel
      this.background = UIManager.getColor("Panel.background");
      this.foreground = UIManager.getColor("Panel.foreground");
      this.selectedBackground = UIManager.getColor("ComboBox.selectionBackground");
      this.selectedForeground = UIManager.getColor("ComboBox.selectionForeground");

      initializePopup();
    }

    //========================================
    // begin ComboPopup method implementations
    //
    public void show() {
      try {
        // if setSelectedItem() was called with a valid date, adjust the calendar
        java.util.Date dt = this.comboBox.getDate();
        this.calendar.setTime(dt);
      }
      catch (Exception e) {
      }
      updatePopup();
      this.popup.show(this.comboBox, 0, this.comboBox.getHeight());
    }

    public void hide() {
      this.popup.setVisible(false);
    }

    protected JList list = new JList();
    public JList getList() {
      return this.list;
    }

    public MouseListener getMouseListener() {
      return this;
    }

    public MouseMotionListener getMouseMotionListener() {
      return this;
    }

    public KeyListener getKeyListener() {
      return this;
    }

    public boolean isVisible() {
      return this.popup.isVisible();
    }

    public void uninstallingUI() {
      this.popup.removePopupMenuListener(this);
    }

    //
    // end ComboPopup method implementations
    //======================================

    //===================================================================
    // begin Event Listeners
    //

    // MouseListener

    public void mousePressed(MouseEvent e) {
    }
    public void mouseReleased(MouseEvent e) {
    }
    // something else registered for MousePressed
    public void mouseClicked(MouseEvent e) {
      if (!SwingUtilities.isLeftMouseButton(e))
        return;
      if (!this.comboBox.isEnabled())
        return;
      if (this.comboBox.isEditable()) {
        //comboBox.getEditor().getEditorComponent().requestFocus();
      }
      else {
        //comboBox.requestFocus();
      }
      togglePopup();
    }

    protected boolean mouseInside = false;
    public void mouseEntered(MouseEvent e) {
      this.mouseInside = true;
    }
    public void mouseExited(MouseEvent e) {
      this.mouseInside = false;
    }

    // MouseMotionListener
    public void mouseDragged(MouseEvent e) {
    }
    public void mouseMoved(MouseEvent e) {
    }

    // KeyListener
    public void keyPressed(KeyEvent e) {
    }
    public void keyTyped(KeyEvent e) {
    }
    public void keyReleased(KeyEvent e) {
      if (e.getKeyCode() == KeyEvent.VK_SPACE || e.getKeyCode() == KeyEvent.VK_ENTER) {
        togglePopup();
      }
    }

    /**
     * Variables hideNext and mouseInside are used to
     * hide the popupMenu by clicking the mouse in the JComboBox
     */
    public void popupMenuCanceled(PopupMenuEvent e) {
    }
    protected boolean hideNext = false;
    public void popupMenuWillBecomeInvisible(PopupMenuEvent e) {
      this.hideNext = this.mouseInside;
    }
    public void popupMenuWillBecomeVisible(PopupMenuEvent e) {
    }

    //
    // end Event Listeners
    //=================================================================

    //===================================================================
    // begin Utility methods
    //

    protected void togglePopup() {
      if (isVisible() || this.hideNext) {
        hide();
      }
      else {
        show();
      }
      this.hideNext = false;
    }

    //
    // end Utility methods
    //=================================================================

    // Note *** did not use JButton because Popup closes when pressed
    protected JLabel createUpdateButton(final int field, final int amount) {
      final JLabel label = new JLabel();
      final Border selectedBorder = new EtchedBorder();
      final Border unselectedBorder = new EmptyBorder(selectedBorder.getBorderInsets(new JLabel()));
      label.setBorder(unselectedBorder);
      label.setForeground(this.foreground);
      label.addMouseListener(new MouseAdapter() {
        @Override
		public void mouseReleased(MouseEvent e) {
          DatePopup.this.calendar.add(field, amount);
          updatePopup();
        }
        @Override
		public void mouseEntered(MouseEvent e) {
          label.setBorder(selectedBorder);
        }
        @Override
		public void mouseExited(MouseEvent e) {
          label.setBorder(unselectedBorder);
        }
      });
      return label;
    }

    // ***********************************************************************
    // Changed by Klaue to use the selected language
    protected void initializePopup() {
      JPanel header = new JPanel(); // used Box, but it wasn't Opaque
      header.setLayout(new BoxLayout(header, BoxLayout.X_AXIS));
      header.setBackground(this.background);
      header.setOpaque(true);

      JLabel label;
      label = createUpdateButton(Calendar.YEAR, -1);
      label.setText("<<");
      label.setToolTipText(Dictionary.getInstance().getValue("DateComboBoxPrevYear"));

      header.add(Box.createHorizontalStrut(12));
      header.add(label);
      header.add(Box.createHorizontalStrut(12));

      label = createUpdateButton(Calendar.MONTH, -1);
      label.setText("<");
      label.setToolTipText(Dictionary.getInstance().getValue("DateComboBoxPrevMonth"));
      header.add(label);

      this.monthLabel = new JLabel("", SwingConstants.CENTER);
      this.monthLabel.setForeground(this.foreground);
      header.add(Box.createHorizontalGlue());
      header.add(this.monthLabel);
      header.add(Box.createHorizontalGlue());

      label = createUpdateButton(Calendar.MONTH, 1);
      label.setText(">");
      label.setToolTipText(Dictionary.getInstance().getValue("DateComboBoxNextMonth"));
      header.add(label);

      label = createUpdateButton(Calendar.YEAR, 1);
      label.setText(">>");
      label.setToolTipText(Dictionary.getInstance().getValue("DateComboBoxNextYear"));

      header.add(Box.createHorizontalStrut(12));
      header.add(label);
      header.add(Box.createHorizontalStrut(12));

      this.popup = new JPopupMenu();
      this.popup.setBorder(BorderFactory.createLineBorder(Color.black));
      this.popup.setLayout(new BorderLayout());
      this.popup.setBackground(this.background);
      this.popup.addPopupMenuListener(this);
      this.popup.add(BorderLayout.NORTH, header);
    }

    // update the Popup when either the month or the year of the calendar has been changed
    protected void updatePopup() {
      this.monthLabel.setText(this.monthFormat.format(this.calendar.getTime()));
      if (this.days != null) {
        this.popup.remove(this.days);
      }
      this.days = new JPanel(new GridLayout(0, 7));
      this.days.setBackground(this.background);
      this.days.setOpaque(true);

      Calendar setupCalendar = (Calendar) this.calendar.clone();
      setupCalendar.set(Calendar.DAY_OF_WEEK, setupCalendar.getFirstDayOfWeek());
      for (int i = 0; i < 7; i++) {
        int dayInt = setupCalendar.get(Calendar.DAY_OF_WEEK);
        JLabel label = new JLabel();
        label.setHorizontalAlignment(SwingConstants.CENTER);
        label.setForeground(this.foreground);
        
        // changed by Klaue to add Dictionary-function
        if (dayInt == Calendar.SUNDAY) {
          label.setText(Dictionary.getInstance().getValue("DateComboBoxDaySun"));
        }
        else if (dayInt == Calendar.MONDAY) {
          label.setText(Dictionary.getInstance().getValue("DateComboBoxDayMon"));
        }
        else if (dayInt == Calendar.TUESDAY) {
          label.setText(Dictionary.getInstance().getValue("DateComboBoxDayTue"));
        }
        else if (dayInt == Calendar.WEDNESDAY) {
          label.setText(Dictionary.getInstance().getValue("DateComboBoxDayWed"));
        }
        else if (dayInt == Calendar.THURSDAY) {
          label.setText(Dictionary.getInstance().getValue("DateComboBoxDayThu"));
        }
        else if (dayInt == Calendar.FRIDAY) {
          label.setText(Dictionary.getInstance().getValue("DateComboBoxDayFri"));
        }
        else if (dayInt == Calendar.SATURDAY) {
          label.setText(Dictionary.getInstance().getValue("DateComboBoxDaySat"));
        }
        this.days.add(label);
        setupCalendar.roll(Calendar.DAY_OF_WEEK, true);
      }

      setupCalendar = (Calendar) this.calendar.clone();
      setupCalendar.set(Calendar.DAY_OF_MONTH, 1);
      int first = setupCalendar.get(Calendar.DAY_OF_WEEK);
     
      // ********************************************************************
      // Klaue: corrected a bug that occurs if Calendar.getFirstDayOfWeek() is not 1
      // like it is in the USA
      for (int i = setupCalendar.getFirstDayOfWeek(); i < first; i++) {
        this.days.add(new JLabel(""));
      }
      for (int i = 1; i <= setupCalendar.getActualMaximum(Calendar.DAY_OF_MONTH); i++) {
        final int day = i;
        final JLabel label = new JLabel(String.valueOf(day));
        label.setHorizontalAlignment(SwingConstants.CENTER);
        label.setForeground(this.foreground);
        label.addMouseListener(new MouseListener() {
          public void mousePressed(MouseEvent e) {
          }
          public void mouseClicked(MouseEvent e) {
          }
          public void mouseReleased(MouseEvent e) {
            label.setOpaque(false);
            label.setBackground(DatePopup.this.background);
            label.setForeground(DatePopup.this.foreground);
            DatePopup.this.calendar.set(Calendar.DAY_OF_MONTH, day);
            DatePopup.this.comboBox.setDate(DatePopup.this.calendar.getTime());
            //comboBox.setSelectedItem(dateFormat.format(calendar.getTime()));
            hide();
            // hide is called with setSelectedItem() ... removeAll()
            //comboBox.getEditor().getEditorComponent().requestFocus();
          }
          public void mouseEntered(MouseEvent e) {
            label.setOpaque(true);
            label.setBackground(DatePopup.this.selectedBackground);
            label.setForeground(DatePopup.this.selectedForeground);
          }
          public void mouseExited(MouseEvent e) {
            label.setOpaque(false);
            label.setBackground(DatePopup.this.background);
            label.setForeground(DatePopup.this.foreground);
          }
        });

        this.days.add(label);
      }

      this.popup.add(BorderLayout.CENTER, this.days);
      this.popup.pack();
    }
  }

  //////////////////////////////////////////////////////////////
  // This is only included to provide a sample GUI
  //////////////////////////////////////////////////////////////
  /*public static void main(String args[]) {
    JFrame f = new JFrame();
    Container c = f.getContentPane();
    c.setLayout(new FlowLayout());
    c.add(new JLabel("Date 1:"));
    DateComboBox box = new DateComboBox();
    box.setName("One");
    c.add(box);
    c.add(new JLabel("Date 2:"));
    DateComboBox dcb = new DateComboBox();
    dcb.setName("Two");
    c.add(dcb);
    f.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        System.exit(0);
      }
    });

    try {
      String sLookAndFeel = "com.sun.java.swing.plaf.windows.WindowsLookAndFeel";
      UIManager.setLookAndFeel(sLookAndFeel);
      SwingUtilities.updateComponentTreeUI(f);
    }
    catch (Exception ex) {
    }

    f.setSize(500, 200);
    f.setVisible(true);
  }*/

}

class DateFocusListener implements FocusListener {

	  private Object prevSelection = null;
	  DateComboBox parent = null;
	  
//	  private static final long LOST_INTERVAL = 50;//50 millis.

	  public DateFocusListener(DateComboBox parent) {
	    this.parent = parent;
	  }


	  public void focusGained(FocusEvent ev){
	    this.prevSelection = this.parent.getSelectedItem();
	    JTextField fld = (JTextField)this.parent.getEditor().getEditorComponent();
	    fld.selectAll();
	  }

	  public void focusLost(FocusEvent ev){
	    
	    //Check if user entered a valid date according to pattern. If so,
	    //update the selected value. If not, reset to previous value???
	    JTextField fld = (JTextField)this.parent.getEditor().getEditorComponent();
	    String enteredText = fld.getText();
	    if (enteredText != null && ! enteredText.trim().equals("")){
	      try{
	        String sDate = enteredText.trim();
	        java.util.Date date = this.parent.parseDateString(sDate);
	        this.parent.setDate(date);
	      }
	      catch (java.text.ParseException ex){
	        //Invalid date. Update text field with previous value.
	        this.parent.setSelectedItem(this.prevSelection);
	      }
	    }
	    else{
	      //Null date.
	      this.parent.setSelectedItem(null);
	    }
	  }
}