package Helpers;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.Vector;

public class Dictionary {
	private static Dictionary ref;
	
	private String m_sLang = "English";
	private String m_sLongestLang = null;
	
	private File fDirectoryPath = new File("Workfiles" + File.separator + "languages");
	private TreeSet<String> mLanguages = new TreeSet<String>();
	private Vector<String> vLangFiles = new Vector<String>();
	TreeMap<String, TreeMap<String, String> > mStrings = new TreeMap<String, TreeMap<String, String> >();

	private Dictionary() {
		File[] files = this.fDirectoryPath.listFiles();
		for (int i = 0; i < files.length; ++i) {
			if (!files[i].isDirectory ()) {
				if (files[i].getName().substring(files[i].getName().length() - 4).equals(".lng")) {
					String sActualFile = files[i].getName();
					sActualFile = sActualFile.substring(0, sActualFile.length() - 4);
					this.mLanguages.add(sActualFile);
					this.vLangFiles.add(files[i].getAbsolutePath());
				}
			}
		}
		
		for (int i = 0; i < this.vLangFiles.size(); ++i) {
			try {
		        //BufferedReader in = new BufferedReader(new FileReader(vLangFiles.elementAt(i)));
				BufferedReader in = new BufferedReader(new InputStreamReader(new FileInputStream(this.vLangFiles.elementAt(i)), "UTF-8"));
		        String str;
		        String sLang = this.vLangFiles.elementAt(i);
		        sLang = sLang.substring(0, sLang.length() - 4);
		        sLang = sLang.substring(sLang.lastIndexOf(File.separator) + 1, sLang.length()).trim();
		        TreeMap<String, String> mNameValues = new TreeMap<String, String>();
		        while ((str = in.readLine()) != null) {
		        	str = str.trim();
		        	if (str.indexOf("=") == -1) continue;
		            String sName = str.substring(0, str.indexOf("=", 0)).trim();
		            String sValue = str.substring(str.indexOf("=", 0) + 1, str.length()).trim();
		            mNameValues.put(sName, sValue);
		        }
		        this.mStrings.put(sLang, mNameValues);
		        in.close();
		    } catch (IOException e) {
		    	e.printStackTrace();
		    	e.printStackTrace(Logwriter.getInstance().getPrintStream());
		    }
		}
	}

	public static synchronized Dictionary getInstance()
    {
      if (ref == null) {
          ref = new Dictionary();
      }
      return ref;
    }
	
	//just to be sure
	@Override
	public Object clone() throws CloneNotSupportedException {
	    throw new CloneNotSupportedException(); 
	}
	
	public String getValue(String sName) {
		String sReturn = this.mStrings.get(this.m_sLang).get(sName);
		if (sReturn == null) {
			System.err.println("Language " + this.m_sLang + " or Name " + sName + " not in language database");
			System.exit(1);
		}
		return sReturn;
	}
	
	public void setLanguage(String sLanguage) {
		this.m_sLang = sLanguage;
	}
	
	public String getLanguage() {
		return this.m_sLang;
	}
	
	public String getLongestLanguage() {
		if (this.m_sLongestLang == null) {
			this.m_sLongestLang = "";
			Iterator<String> iter = this.mLanguages.iterator();
			while (iter.hasNext()) {
				String s = iter.next();
				if (this.m_sLongestLang.length() < s.length()) {
					this.m_sLongestLang = s;
				}
			}
		}
		return this.m_sLongestLang;
	}
	
	public String[] getLanguages() {
		String[] sLanguages = new String[this.mLanguages.size()];
		Iterator<String> iter = this.mLanguages.iterator();
		int i = 0;
		while (iter.hasNext()) {
			sLanguages[i] = iter.next();
			++i;
		}
		return sLanguages;
	}

}
