package Helpers;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.TreeMap;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import Helpers.TreeTable.AbstractTreeTableModel;
import Helpers.TreeTable.JTreeTable;
import Helpers.TreeTable.TreeTableModel;

public class ErrorLogger {
	private static ErrorLogger ref = null;
	private TreeMap<String, TreeMap<String, Vector<TreeMap<Type, String> > > > errorMap;
	
	private ErrorLogger() {
		this.errorMap = new TreeMap<String, TreeMap<String, Vector<TreeMap<Type, String> > > >();
	}

	public static synchronized ErrorLogger getInstance() {
		if (ref == null) {
			ref = new ErrorLogger();
		}
		return ref;
    }
	
	public void addError(String sPage, String sUser, String sUrl, String sErrorMessage) {
		TreeMap<Type, String> tempMap = new TreeMap<Type, String>();
		if (sUrl != null) tempMap.put(Type.URL, sUrl);
		tempMap.put(Type.MESSAGE, sErrorMessage);
		
		if (!this.errorMap.containsKey(sPage)) {
			this.errorMap.put(sPage, new TreeMap<String, Vector<TreeMap<Type, String> > >());
		}
		if (!this.errorMap.get(sPage).containsKey(sUser)) {
			this.errorMap.get(sPage).put(sUser, new Vector<TreeMap<Type, String> >());
		}
		this.errorMap.get(sPage).get(sUser).add(tempMap);
	}
	
	public void reset() {
		this.errorMap.clear();
	}
	
	public TreeMap<String, TreeMap<String, Vector<TreeMap<Type, String> > > > getErrors() {
		return this.errorMap;
	}
	
	public void showErrors(JFrame parent) {
		final JDialog dialog = new JDialog(parent);
		dialog.setTitle(Dictionary.getInstance().getValue("ErrorDownloadingTitle"));
		JPanel pnlContent = new JPanel();
		pnlContent.setLayout(new BoxLayout(pnlContent, BoxLayout.Y_AXIS));
		
		JLabel lblCaption = new JLabel(Dictionary.getInstance().getValue("ErrorText"));
		pnlContent.add(lblCaption);
		pnlContent.add(Box.createVerticalStrut(10));
		
		JTreeTable table = new JTreeTable(new ErrorTreeTableModel(this));
		table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		table.getColumnModel().getColumn(0).setPreferredWidth(200);
		table.getColumnModel().getColumn(1).setPreferredWidth(150);
		table.getColumnModel().getColumn(2).setPreferredWidth(300);
		pnlContent.add(new JScrollPane(table));
		
		pnlContent.add(Box.createVerticalStrut(10));
		JButton btnOK = new JButton("OK");
		btnOK.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				// close (and get rid of) dialog
				dialog.setVisible(false);
				dialog.dispose();
			}
		});
		JPanel buttonPanel = new JPanel();
		buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
		buttonPanel.add(Box.createHorizontalGlue());
		buttonPanel.add(btnOK);
		buttonPanel.add(Box.createHorizontalGlue());
		pnlContent.add(buttonPanel);
		pnlContent.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10)); 
		dialog.add(pnlContent);
		
		//JTreeTable treeTable = new JTreeTable(new ErrorTreeTableModel(this));
		dialog.setSize(600, 330);
		//dialog.validate();
		dialog.setLocationRelativeTo(parent);
		dialog.setModal(true);
		dialog.setVisible(true);

	}
	
	private enum Type {
	    MESSAGE, URL 
	}
	
	private class ErrorTreeTableModel  extends AbstractTreeTableModel {
		private ErrorLogger errorLogger = null;
		@SuppressWarnings("unchecked")
		private final Class[]  columnTypes = {TreeTableModel.class, String.class, String.class};
		// I'm too lazy to translate that right now
		private String[]  columnTitles = {Dictionary.getInstance().getValue("ErrorLoggerPage"),
										Dictionary.getInstance().getValue("ErrorLoggerURL"),
										Dictionary.getInstance().getValue("ErrorLoggerError")};
		
		public ErrorTreeTableModel(ErrorLogger errorLogger) {
			super(new MyNode(errorLogger, null, null, -1));
			this.errorLogger = errorLogger;
		}
		
		@SuppressWarnings("unchecked")
		@Override
		public Class getColumnClass(int column) {
			return this.columnTypes[column];
		}

		public int getColumnCount() {
			return 3;
		}

		public String getColumnName(int column) {
			return this.columnTitles[column];
		}

		public Object getValueAt(Object node, int column) {
			MyNode myNode = (MyNode)node;
			switch(column) {
		    	case 0:
	    			return myNode;
		    	case 1:
	    			if (myNode.page == null || myNode.user == null || myNode.index == -1) {
	    				return null;
	    			}
					return this.errorLogger.getErrors().get(myNode.page).get(myNode.user).get(myNode.index).get(Type.URL);
		    	case 2:
	    			if (myNode.page == null || myNode.user == null || myNode.index == -1) {
	    				return null;
	    			}
					return this.errorLogger.getErrors().get(myNode.page).get(myNode.user).get(myNode.index).get(Type.MESSAGE);
    			default:
    				return null;
		    }
		}

		@Override
		public boolean isCellEditable(Object node, int column) {
			// TODO Auto-generated method stub
			return true;
		}

		@Override
		public void setValueAt(Object aValue, Object node, int column) {
			return;
		}

		@Override
		public Object getChild(Object node, int i) {
			return ((MyNode)node).getChildren()[i];
		}

		@Override
		public int getChildCount(Object node) {
			if (((MyNode)node).getChildren() == null) return 0;
			return ((MyNode)node).getChildren().length;
		}

		@Override
		public boolean isLeaf(Object node) {
			return (getChildCount(node) == 0);
		}
	}

	private class MyNode {
	    String page = null;
	    String user = null;
	    int index; 
	    ErrorLogger errorLogger;
	    Object[] children = null;

	    public MyNode(ErrorLogger errorLogger, String page, String user, int index) {
	    	this.page = page;
	    	this.user = user;
	    	this.index = index;
	    	this.errorLogger = errorLogger;
	    }

	    /**
	     * Returns the the string to be used to display this leaf in the JTree.
	     */
	    @Override
		public String toString() {
	    	if (this.page != null) {
	    		if (this.user != null) {
	    			if (this.index != -1) {
	    				return Dictionary.getInstance().getValue("ErrorLoggerError") + this.index;
	    			}
	    			return this.user;
	    		}
	    		return this.page;
	    	}
	    	return "";
	    }

	    /**
	     * Loads the children, caching the results in the children ivar.
	     */
	    protected Object[] getChildren() {
	    	if (this.page != null) {
	    		if (this.user != null) {
	    			if (this.index != -1) {
	    				return null;
	    			}
					this.children = new MyNode[this.errorLogger.getErrors().get(this.page).get(this.user).size()];
					for (int i = 0; i < this.errorLogger.getErrors().get(this.page).get(this.user).size(); ++i) {
						this.children[i] = new MyNode(this.errorLogger, this.page, this.user, i);
					}
					return this.children;
	    		}
				this.children = new MyNode[this.errorLogger.getErrors().get(this.page).size()];
				Iterator<String> iter = this.errorLogger.getErrors().get(this.page).keySet().iterator();
				int i = 0;
				while (iter.hasNext()) {
					this.children[i] = new MyNode(this.errorLogger, this.page, iter.next(), -1);
					++i;
				}
				return this.children;
	    	}
			this.children = new MyNode[this.errorLogger.getErrors().size()];
			Iterator<String> iter = this.errorLogger.getErrors().keySet().iterator();
			int i = 0;
			while (iter.hasNext()) {
				this.children[i] = new MyNode(this.errorLogger, iter.next(), null, -1);
				++i;
			}
			return this.children;
	    }
	}
}
