package Helpers;
import java.util.TreeSet;

import DownloadHelpers.WrongUsernameOrPasswordException;

public interface Grabbable {
	// get (download) the images from this account. return them in a set of the images.
	// if there are no new images for the user since the date&time in the account,  return an empty set.
	// if an error occured, return null
	// the date&time in the accounts will not exceed the current date&time, so you don't need to check it
	// but remember that the date&time in the account is the one of the user's time zone, so make
	// sure you use the time zone of the page (see http://java.sun.com/j2se/1.5.0/docs/api/java/util/TimeZone.html
	// and http://java.sun.com/j2se/1.5.0/docs/api/java/util/Calendar.html#setTimeZone(java.util.TimeZone) )
	// be carfull with DST (summertime for brits). if you don't know if it's curerently summertime at the page, better
	// take the pictures of one hour earlier than later
	
	// The given String is the saveplace of the images. every page and account will have its own folder.
	// the names of the images are up to you
	// tip: use the current milliseconds of the hour and add the original filename, so you can be sure
	// that your image will not be overwritten and the user gets the real filename too
	
	// the userNotFoundImg-GrabberImage is an image to use instead of one in the target page if
	// the target user was not found. For example if the target-username is "Goofie" and there is no
	// user with this name on the page (you get an 404 or something like this) then use that:
	// TreeSet<GrabberImage> set = new TreeSet<GrabberImage>();
	// set.add(userNotFoundImg);
	// return set
	public TreeSet<GrabberImage> getImages(Account acc, String sSavePath, GrabberImage userNotFoundImg);
	
	// login to the site / "click" the "yes, i'm old enough" or something like this
	// if any error occures while this, return false
	// if username/password is wrong, throw an WrongUsernameOrPasswordException
	// if your page doesn't need it, just leave it empty and return true
	public boolean doLogin(Account acc) throws WrongUsernameOrPasswordException;
	
	// verify if an account belongs to this site
	public boolean isFromThisSite(Account acc);
	
	// needs an username no matter what (means there's no way to get anything withoutr an username)
	public boolean needsUsernameForAllContent();
	
	// needs an password no matter what (means there's no way to get anything withoutr an password)
	public boolean needsPasswordForAllContent();
	
	// needs an username only for special content (e.g. adult stuff)
	public boolean needsUsernameForSpecialContent();
	
	// needs an password only for special content (e.g. adult stuff)
	public boolean needsPasswordForSpecialContent();
	
	// has only a fixed set of possible target user accounts (or subsites in chans)
	public boolean fixedTargetUsers();
	
	// return the fixed set of target users. if fixedTargetUsers() returns false, this method can return null
	public String[] getFixedTargetUsers();
	
	// return the string that describes what the user should enter in the target user field
	// (e.g. "Zu verfolgender Benutzer" or "Bereich des Chans"). should be as short as possible
	// and in the language of the site
	public String getTargetUserDescription();
	
	// return an short string which is the title of the site. like "DeviantArt" or "FChan"
	public String getSitename();
	
	// return a string with an description of the site. add also if the user should give an username
	// or password and why. and if he doesnt have to give one (see needsPasswordForSpecialContent())
	// then describe what he misses if he discards the username/password
	// newlines are added when needed
	// description should be in the same language as the page
	public String getDescription();
}
