package Helpers;

import java.util.Iterator;
import java.util.Vector;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/** 
 * MySwing: Advanced Swing Utilites 
 * Copyright (C) 2005  Santhosh Kumar T 
 * <p/> 
 * This library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation; either 
 * version 2.1 of the License, or (at your option) any later version. 
 * <p/> 
 * This library is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 * Lesser General Public License for more details. 
 */ 
public class ProgressMonitor{ 
    int total, current=-1; 
    boolean indeterminate; 
    int milliSecondsToWait = 500; // half second 
    String status; 
    public ProgressMonitor(int total, boolean indeterminate, int milliSecondsToWait){ 
        this.total = total; 
        this.indeterminate = indeterminate; 
        this.milliSecondsToWait = milliSecondsToWait; 
    } 
 
    public ProgressMonitor(int total, boolean indeterminate){ 
        this.total = total; 
        this.indeterminate = indeterminate; 
    } 
 
    public int getTotal(){ 
        return this.total; 
    } 
 
    public void start(String status){ 
        if(this.current!=-1) 
            throw new IllegalStateException("not started yet"); 
        this.status = status; 
        this.current = 0; 
        fireChangeEvent(); 
    } 
 
    public int getMilliSecondsToWait(){ 
        return this.milliSecondsToWait; 
    } 
 
    public int getCurrent(){ 
        return this.current; 
    } 
 
    public String getStatus(){ 
        return this.status; 
    } 
 
    public boolean isIndeterminate(){ 
        return this.indeterminate; 
    } 
 
    public void setCurrent(String status, int current){ 
        if(current==-1) 
            throw new IllegalStateException("not started yet"); 
        this.current = current; 
        if(status!=null) 
            this.status = status; 
        fireChangeEvent(); 
    } 
 
    /*--------------------------------[ ListenerSupport ]--------------------------------*/ 
 
    private Vector<ChangeListener> listeners = new Vector<ChangeListener>(); 
    private ChangeEvent ce = new ChangeEvent(this); 
 
    public void addChangeListener(ChangeListener listener){ 
        this.listeners.add(listener); 
    } 
 
    public void removeChangeListener(ChangeListener listener){ 
        this.listeners.remove(listener); 
    } 
 
    private void fireChangeEvent(){ 
        Iterator<ChangeListener> iter = this.listeners.iterator(); 
        while(iter.hasNext()){ 
            iter.next().stateChanged(this.ce); 
        } 
    } 
}