package Sites;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.TimeZone;
import java.util.TreeSet;
import java.util.regex.Matcher;

import klaue.DownloadHelper.Downloader;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.JSONValue;

import DownloadHelpers.WrongUsernameOrPasswordException;
import Helpers.Account;
import Helpers.Dictionary;
import Helpers.ErrorLogger;
import Helpers.Grabbable;
import Helpers.GrabberImage;
import Helpers.Logwriter;

/**
 * @author klaue
 *
 */
public class DeviantArt implements Grabbable {
	final static String PAGENAME = "DeviantArt";
	final static String LOGIN_PGE = "https://www.deviantart.com/users/login";
	final static String ARTIST_URL = ".deviantart.com/";
	final static String GALLERY_URL_PRE = ".deviantart.com/global/difi/?c[]=\"Resources\",\"htmlFromQuery\",[\"gallery:";
	final static String GALLERY_URL_GAL = " sort:time\","; // followed by start,end (0,24)
	final static String GALLERY_URL_SCRAP = "  sort:time in:scraps\","; // followed by start,end (0,24)
	final static String GALLERY_URL_END = ",\"thumb150\",\"artist:0\"]&t=json";
	String TIMEZONE = "";
	
	GrabberImage m_fourOhFourImg = null;
	String m_ImageFolder = "";

	@Override
	public TreeSet<GrabberImage> getImages(Account acc, String sSavePath, GrabberImage userNotFoundImg) {
		if (acc == null) return null;
		this.m_ImageFolder = sSavePath;
		this.m_fourOhFourImg = userNotFoundImg;
		TreeSet<GrabberImage> returnSet = new TreeSet<GrabberImage>();
		
		// checking if user exists
		URL url = null;
		try {
			url = new URL("http://" + acc.TargetUser + DeviantArt.ARTIST_URL);
			Downloader.downloadPage(url, null, null, 20000, 3);
		} catch (MalformedURLException e) {
			TreeSet<GrabberImage> mReturnSet = new TreeSet<GrabberImage>();
			mReturnSet.add(userNotFoundImg);
			return mReturnSet;
		} catch (FileNotFoundException e) {
			TreeSet<GrabberImage> mReturnSet = new TreeSet<GrabberImage>();
			mReturnSet.add(userNotFoundImg);
			return mReturnSet;
		} catch (IOException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
		
		// getting gallery
		TreeSet<GrabberImage> tmpSet = null;
		tmpSet = getImagesFromAccount("http://" + acc.TargetUser + DeviantArt.GALLERY_URL_PRE + acc.TargetUser + DeviantArt.GALLERY_URL_GAL, 0, DeviantArt.GALLERY_URL_END, acc.getDate(), acc.TargetUser);
		if (tmpSet == null) return null;
		returnSet.addAll(tmpSet);
		
		// getting scraps
		tmpSet = getImagesFromAccount("http://" + acc.TargetUser + DeviantArt.GALLERY_URL_PRE + acc.TargetUser + DeviantArt.GALLERY_URL_SCRAP, 0, DeviantArt.GALLERY_URL_END, acc.getDate(), acc.TargetUser);
		if (tmpSet != null) returnSet.addAll(tmpSet);
		return returnSet;
	}

	@Override
	public boolean doLogin(Account acc) throws WrongUsernameOrPasswordException {
		if (acc.getPassword().equals("") || acc.getUser().equals("")) {
			// user diddnt set an username/pass, so no logging in. poor user will not get any
			// smut *harrharr*
			return true;
		}
		
		// get hidden values to authentificate from the login page
		ArrayList<String> loginPage;
		try {
			loginPage = Downloader.downloadPage(new URL(DeviantArt.LOGIN_PGE), null, null, 20000, 3);
		} catch (Exception e) {
			// not much we can do
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return false;
		}
		int start = 0;
		int end = 0;
		HashMap<String, String> valuesMap = new HashMap<String, String>(); // to keep values from being doubled (validates are in there twice)
		for (String line : loginPage) {
			String param = null;
			if (line.contains("name=\"ref\"") && !line.contains("loginbar-ref")) {
				// loginbar-ref is a honeypot
				param = "ref";
			}
			if (line.contains("name=\"validate_key\"")) {
				param = "validate_key";
			}
			if (line.contains("name=\"validate_token\"")) {
				param = "validate_token";
			}
			
			if (param != null) {
				start = line.indexOf("value=\"") + 7;
				end = line.indexOf("\"", start);
				valuesMap.put(param, line.substring(start, end));
			}
		}
		
		StringBuffer postParams = new StringBuffer();
		postParams.append("username=").append(acc.getUser()).append("&password=").append(acc.getPassword()).append("&remember_me=0&reusetoken=1");
		for (String param : valuesMap.keySet()) {
			postParams.append("&").append(param).append("=").append(valuesMap.get(param));
		}
		
		ArrayList<String> answer = null;
		try {
			ArrayList<String> test = Downloader.sendPOST(new URL(DeviantArt.LOGIN_PGE), postParams.toString(), DeviantArt.LOGIN_PGE, null, 20000, 3);
			for (String s : test) {
				if (s.indexOf("Click here to continue to deviantART") != -1) {
					Downloader.sendPOST(new URL(DeviantArt.LOGIN_PGE), postParams.toString(), DeviantArt.LOGIN_PGE, null, 20000, 3);
					break;
				}
			}
		} catch (MalformedURLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return false;
		} catch (IOException e) {
			String sErrMsg = e.getMessage();
			
			// because dA returns an Error "302 redirect" to the wrong-pass-site,
			// we have to catch this error here. Usually java would just follow the redirect, but it's to a differnet protocol https->http
			if (sErrMsg.indexOf("http://www.deviantart.com/users/wrong-password") != -1) {
				throw new WrongUsernameOrPasswordException(acc.getUser(), acc.getPassword());
			}
			
			// and the success page is also a 302 
			if (sErrMsg.indexOf("HTTP response code: 302") == -1) {
				e.printStackTrace();
				e.printStackTrace(Logwriter.getInstance().getPrintStream());
				return false;
			}
		}
		
		// now we have to check if we are really logged in (e.g. if we're able to connect to http://my.deviantart.com/settings/)
		// we take this disadvantage for our own profit and use it to get the timezone ;)
		try {
			answer = Downloader.downloadPage(new URL ("https://www.deviantart.com/settings/index"), null, null, 20000, 3);
			for (String s : answer) {
				if (s.indexOf("Click here to continue to deviantART") != -1) {
					answer = Downloader.downloadPage(new URL ("http://my.deviantart.com/settings/"), null, null, 20000, 3);
					break;
				}
			}
			
			boolean bInTimezone = false;
			for (String sLine : answer) {
				if (sLine.indexOf("<title>") != -1) {
					if (sLine.indexOf("Login Required") != -1) {
						//we're not logged in
						return false;
					}
					// hurray, we're logged in!
				}

				if (sLine.indexOf("id=\"timezone_id\"") != -1) bInTimezone = true;
				if (bInTimezone && sLine.indexOf("</select>") != -1) bInTimezone = false;
				if (bInTimezone && sLine.indexOf("selected=\"selected\"") != -1) {
					int iStart = sLine.indexOf("value=\"") + 7;
					int iEnd = sLine.indexOf("\"", iStart);
					this.TIMEZONE = sLine.substring(iStart, iEnd);
				}
			}
			return true;
		} catch (Exception wtf) {
			wtf.printStackTrace();
			wtf.printStackTrace(Logwriter.getInstance().getPrintStream());
			return false;
		}
	}

	@Override
	public boolean isFromThisSite(Account acc) {
		if (acc.PageName.equals(DeviantArt.PAGENAME)) return true;
		return false;
	}

	@Override
	public boolean needsUsernameForAllContent() {
		return false;
	}

	@Override
	public boolean needsPasswordForAllContent() {
		return false;
	}

	@Override
	public boolean needsUsernameForSpecialContent() {
		return true;
	}

	@Override
	public boolean needsPasswordForSpecialContent() {
		return true;
	}

	@Override
	public boolean fixedTargetUsers() {
		return false;
	}

	@Override
	public String[] getFixedTargetUsers() {
		return null;
	}

	@Override
	public String getTargetUserDescription() {
		return Dictionary.getInstance().getValue("SitesUserDescrDeviantArt");
	}

	@Override
	public String getSitename() {
		return DeviantArt.PAGENAME;
	}

	@Override
	public String getDescription() {
		return Dictionary.getInstance().getValue("SitesDescrDeviantArt");
	}
	
	private TreeSet<GrabberImage> getImagesFromAccount(String sUrlFirstPart, int iStartIdx, String sUrlSecondPart, Date date, String sTargetUser) {
		// Build the url
		URL mainUrl;
		try {
			mainUrl = new URL(sUrlFirstPart + iStartIdx + "," + (iStartIdx + 24) + sUrlSecondPart);
		} catch (MalformedURLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
		
		TreeSet<GrabberImage> set = new TreeSet<GrabberImage>();
		boolean nextPage = false;
		try {
			ArrayList<String> page = Downloader.downloadPage(mainUrl, null, null, 20000, 3);
			for (String s : page) {
				if (s.indexOf("Click here to continue to deviantART") != -1) {
					page = Downloader.downloadPage(mainUrl, null, null, 20000, 3);
					break;
				}
			}
			String sJSON = page.get(0);

			JSONObject jsonObject = (JSONObject)JSONValue.parse(sJSON);
			jsonObject = (JSONObject)jsonObject.get("DiFi");
			if (!((String)jsonObject.get("status")).equals("SUCCESS")) {
				return null;
			}
			
			jsonObject = (JSONObject)((JSONObject)((JSONArray)((JSONObject)jsonObject.get("response")).get("calls")).get(0)).get("response");
			if (!((String)jsonObject.get("status")).equals("SUCCESS")) {
				return null;
			}
			
			jsonObject = (JSONObject)jsonObject.get("content");
			nextPage = (Boolean)jsonObject.get("more");
			
			JSONArray imageArray = (JSONArray)jsonObject.get("resources");
			
			for (int i = 0; i < imageArray.size(); ++i) {
				String sImageData = (String)((JSONArray)imageArray.get(i)).get(2);
				// string could be empty
				if (sImageData.trim().isEmpty()) continue;
				
				// in the past, stuff between <!-- ^TTT --> and its end tag was used. seems that now dA doesn't neccesarly set those tags anymore
				// but has other ones - an <a tag filled with all the info we need
				// Example:
				// <a super_img="http://fc03.deviantart.net/fs50/i/2009/321/0/4/Reaching_Out_For_Love_by_darklonleyheart.jpg"
				//    super_w="900"
				//    super_h="563"
				//    super_catid="1487"
				//    href="http://darklonleyheart.deviantart.com/art/Reaching-Out-For-Love-143922244?
				//          q=gallery%3Adarklonleyheart+sort%3Atime&amp;qo=0"
				//    title="Reaching Out For Love by ~darklonleyheart, Nov 17, 2009 in&nbsp;Digital&nbsp;
				//           Art&nbsp;>&nbsp;Photomanipulation&nbsp;>&nbsp;Dark"
				// >
				// note: we can't use the super_img tag, because sometimes the image aviable through the download button
				// is larger that that.
				int iStart = sImageData.indexOf("<a");
				sImageData = sImageData.substring(iStart, sImageData.indexOf("\">", iStart) + 2);
				
				// if there is no title tag, the deviation is text (a poem or so) and not supported by GG, skip it
				if (sImageData.indexOf("title=\"") == -1) continue;
				
				// if there is no href tag, the deviation is "in storage", whatever that means for dA. anyway, we can't get it
				if (sImageData.indexOf("href=\"") == -1) continue;
				
				// get title (everything in title tag in front of "by <symbol><user>
				// (this is a bit ugly, but I hate regex and want to get over with it)
				iStart = sImageData.indexOf("title=\"") + 7;
				String sTitleLine = sImageData.substring(iStart, sImageData.indexOf("\"", iStart + 1));
				// q&d fix for special user symbol ° (&#176;)
				sTitleLine = sTitleLine.replaceAll("&#176;", "°");
				// (?i) = ignore case, \Q - \E = ignore regex stuff
				sTitleLine = sTitleLine.replaceAll(" (?i)\\Q" + sTargetUser.toLowerCase() + "\\E", Matcher.quoteReplacement(" %" + sTargetUser));
				String sTitle = sTitleLine.substring(0, sTitleLine.indexOf(" by %" + sTargetUser));
				
				// get date (after username in title)
				iStart = sImageData.indexOf("title=\"") + 7;
				iStart = sTitleLine.lastIndexOf(sTargetUser + ",") + sTargetUser.length() + 1;
				String sDate = sTitleLine.substring(iStart, sTitleLine.indexOf(" in", iStart)).trim();
				DateFormat df = new SimpleDateFormat("MMM d, yyyy", Locale.US);
				df.setTimeZone(TimeZone.getTimeZone(this.TIMEZONE));
				Date imageDate = df.parse(sDate);
				if (date.after(imageDate)) {
					// image too old, abort
					nextPage = false;
					break;
				}

				iStart = sImageData.indexOf("href=\"") + 6;
				String sImageUrl = sImageData.substring(iStart, sImageData.indexOf("\"", iStart + 1));
				try {
					set.add(getImage(new URL(sImageUrl), imageDate, sTitle, sTargetUser));
				} catch (DownloadException e) {
					// just continue to next image
					continue;
				}
			}
			
			
		} catch (Exception e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
		
		if (nextPage) {
			TreeSet<GrabberImage> tmpSet = getImagesFromAccount(sUrlFirstPart, iStartIdx + 24, sUrlSecondPart, date, sTargetUser);
			if (tmpSet == null) return null;
			set.addAll(tmpSet);
		}
		return set;
	}

	private GrabberImage getImage(URL url, Date dat, String title, String sTargetUser) throws DownloadException {
		return getImage(url, dat, title, sTargetUser, this.m_ImageFolder, DeviantArt.PAGENAME);
	}
	
	/**
	 * additional parameters for DeviantArtFavs
	 * @param url
	 * @param dat
	 * @param title
	 * @param sTargetUser
	 * @param imageFolder
	 * @param pageName
	 * @return the grabberImage
	 * @throws DownloadException
	 */
	protected static GrabberImage getImage(URL url, Date dat, String title, String sTargetUser, String imageFolder, String pageName) throws DownloadException {
		
		/*
		 * NOTE for future klaue: the only way difi would allow for comments is using getArtistComments here: https://github.com/danopia/deviantart-difi/wiki/Comments
		 * sadly, it claims that method does not exist. Also, while dA's oembed (http://www.deviantart.com/developers/oembed) would
		 * work to get the real url of submission images, it does not work for flash submissions (returns preview images of them)
		 */
		
		GrabberImage img = null;
		try {
			String sDescr = "";
			String sLocalImgPath = "";
			
			ArrayList<String> imgPage = Downloader.downloadPage(url, null, null, 20000, 3);
			// if ad site, try downbloading once more (then ignore)
			for (String s : imgPage) {
				if (s.indexOf("Click here to continue to deviantART") != -1) {
					imgPage = Downloader.downloadPage(url, null, null, 20000, 3);
					break;
				}
			}
			// first reform to single string. dA likes to throw in linebreaks everywhere so that is more secure
			StringBuffer buffer = new StringBuffer();
			for (String s : imgPage) {
				buffer.append(s.trim());
			}
			String page = buffer.toString();
			// NOTE: page will be quite different when viewed in a browser with JS enabled
			
			// determine type of submission
			boolean isImage = false;
			boolean isFlash = false;
			if (page.contains("class=\"dev-content-full\"")) {
				isImage = true;
			} else if (page.contains("flashtime")) {
				isFlash = true;
			} else {
				// pdfs could be gotten by looking for PDFObjects, but why bother, just display no image
				System.err.println("not supported file type for " + url);
				System.err.println(page);
			}
			
			
			int start = 0;
			int end = 0;
			
			if (isImage) {
				// image
				// get full image url
				start = page.indexOf("class=\"dev-content-full\"");
				// search backwards to get start of img
				start = page.lastIndexOf("<img", start);
				// search forward again to get image url
				start = page.indexOf("src=\"", start) + 5;
				end = page.indexOf("\"", start);
				
				String sImgPath = page.substring(start, end);
				
				sLocalImgPath = imageFolder + sImgPath.substring(sImgPath.lastIndexOf("/") + 1);
				sLocalImgPath = Downloader.downloadFile(new URL(sImgPath), sLocalImgPath, url.toString(), null, 20000, 3);
			}
			if (isFlash) {
				// flash
				// flash is tricky, dA puts it in an iframe, which is really a sandbox url, which we have to call again
				// to get the real flash url
				// note: do not use download button, it uses http redirect which confuses our downloader
				start = page.indexOf("flashtime");
				// search backwards to get start of the iframe
				start = page.lastIndexOf("<iframe", start);
				// search forward again to get flash url
				start = page.indexOf("src=\"", start) + 5;
				end = page.indexOf("\"", start);
				String sandboxUrl = page.substring(start, end);
				
				ArrayList<String> sandbox = Downloader.downloadPage(new URL(sandboxUrl), null, null, 20000, 3);
				for (String line : sandbox) {
					if (!line.contains("sandboxembed")) continue;
					start = line.indexOf("src=\"") + 5;
					end = line.indexOf("\"", start);
					String flashUrl = line.substring(start, end);
					
					String fileName = flashUrl.substring(flashUrl.lastIndexOf("/") + 1);
					sLocalImgPath = imageFolder + fileName;
					
					sLocalImgPath = Downloader.downloadFile(new URL(flashUrl), sLocalImgPath, url.toString(), null, 20000, 3);
					break;
				}
			}

			// get comment
			// first get description start, should be unique
			start = page.indexOf("class=\"dev-description\"");
			// now get the text block from it
			start = page.indexOf("class=\"text block\"", start);
			start = page.indexOf(">", start) + 1;
			end = page.indexOf("</div>", start);
			sDescr = page.substring(start, end);
			
			img = new GrabberImage(sLocalImgPath, title, sDescr, dat, url.toString());
		} catch (Exception e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			String sMessage = e.getLocalizedMessage();
			if (sMessage == null) {
				sMessage = e.toString();
			} else {
				// remove url from message
				if (sMessage.lastIndexOf("http://") != -1) sMessage = sMessage.substring(0, sMessage.lastIndexOf("http://"));
				// remove everything after :
				if (sMessage.lastIndexOf(":") != -1) sMessage = sMessage.substring(0, sMessage.lastIndexOf(":"));
				String sClassdef = e.getClass().toString();
				sMessage = sClassdef.substring(sClassdef.indexOf("class ") + 6) + ((sMessage.length() == 0) ? "" : ": " + sMessage);
			}
			
			ErrorLogger.getInstance().addError(pageName, sTargetUser, url.toString(), "Download failed: " + sMessage);
			throw new DownloadException();
		}
		return img;
	}

}
