package Sites;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;
import java.util.TreeSet;
import java.util.regex.Matcher;

import klaue.DownloadHelper.Downloader;

import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.JSONValue;

import DownloadHelpers.WrongUsernameOrPasswordException;
import Helpers.Account;
import Helpers.Dictionary;
import Helpers.ErrorLogger;
import Helpers.Grabbable;
import Helpers.GrabberImage;
import Helpers.Logwriter;

/**
 * @author klaue
 *
 */
public class DeviantArtFavs implements Grabbable {
	final String PAGENAME = "DeviantArt Favorites";
	final String ARTIST_URL = ".deviantart.com/";
	final String GALLERY_URL_PRE = ".deviantart.com/global/difi/?c[]=\"Resources\",\"htmlFromQuery\",[\"favby:";
	final String GALLERY_URL_GAL = " sort:time\","; // followed by start,end (0,24)
	final String GALLERY_URL_END = ",\"thumb150\",\"artist:0\"]&t=json";
	String TIMEZONE = "";
	
	GrabberImage m_fourOhFourImg = null;
	String m_ImageFolder = "";

	@Override
	public TreeSet<GrabberImage> getImages(Account acc, String sSavePath, GrabberImage userNotFoundImg) {
		if (acc == null) return null;
		this.m_ImageFolder = sSavePath;
		this.m_fourOhFourImg = userNotFoundImg;
		TreeSet<GrabberImage> returnSet = new TreeSet<GrabberImage>();
		
		// checking if user exists
		URL url = null;
		try {
			url = new URL("http://" + acc.TargetUser + this.ARTIST_URL);
			Downloader.downloadPage(url, null, null, 20000, 3);
		} catch (MalformedURLException e) {
			TreeSet<GrabberImage> mReturnSet = new TreeSet<GrabberImage>();
			mReturnSet.add(userNotFoundImg);
			return mReturnSet;
		} catch (FileNotFoundException e) {
			TreeSet<GrabberImage> mReturnSet = new TreeSet<GrabberImage>();
			mReturnSet.add(userNotFoundImg);
			return mReturnSet;
		} catch (IOException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
		
		// getting gallery
		TreeSet<GrabberImage> tmpSet = null;
		tmpSet = getImagesFromAccount("http://" + acc.TargetUser + this.GALLERY_URL_PRE + acc.TargetUser + this.GALLERY_URL_GAL, 0, this.GALLERY_URL_END, acc.getDate(), acc.TargetUser);
		if (tmpSet == null) return null;
		returnSet.addAll(tmpSet);
		
		// getting scraps
		//tmpSet = getImagesFromAccount("http://" + acc.TargetUser + GALLERY_URL_PRE + acc.TargetUser + GALLERY_URL_SCRAP, 0, GALLERY_URL_END, acc.getDate());
		//if (tmpSet != null) returnSet.addAll(tmpSet);
		return returnSet;
	}

	@Override
	public boolean doLogin(Account acc) throws WrongUsernameOrPasswordException {
		DeviantArt daMain = new DeviantArt();
		boolean success = daMain.doLogin(acc);
		this.TIMEZONE = daMain.TIMEZONE;
		return success;
	}

	@Override
	public boolean isFromThisSite(Account acc) {
		if (acc.PageName.equals(this.PAGENAME)) return true;
		return false;
	}

	@Override
	public boolean needsUsernameForAllContent() {
		return false;
	}

	@Override
	public boolean needsPasswordForAllContent() {
		return false;
	}

	@Override
	public boolean needsUsernameForSpecialContent() {
		return true;
	}

	@Override
	public boolean needsPasswordForSpecialContent() {
		return true;
	}

	@Override
	public boolean fixedTargetUsers() {
		return false;
	}

	@Override
	public String[] getFixedTargetUsers() {
		return null;
	}

	@Override
	public String getTargetUserDescription() {
		return Dictionary.getInstance().getValue("SitesUserDescrDeviantArtFav");
	}

	@Override
	public String getSitename() {
		return this.PAGENAME;
	}

	@Override
	public String getDescription() {
		return Dictionary.getInstance().getValue("SitesDescrDeviantArtFav");
	}
	
	private TreeSet<GrabberImage> getImagesFromAccount(String sUrlFirstPart, int iStartIdx, String sUrlSecondPart, Date date, String sTargetUser) {
		// Build the url
		URL mainUrl;
		try {
			mainUrl = new URL(sUrlFirstPart + iStartIdx + "," + (iStartIdx + 24) + sUrlSecondPart);
		} catch (MalformedURLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
		
		TreeSet<GrabberImage> set = new TreeSet<GrabberImage>();
		boolean nextPage = false;
		try {
			ArrayList<String> page = Downloader.downloadPage(mainUrl, null, null, 20000, 3);
			for (String s : page) {
				if (s.indexOf("Click here to continue to deviantART") != -1) {
					page = Downloader.downloadPage(mainUrl, null, null, 20000, 3);
					break;
				}
			}
			String sJSON = page.get(0);

			JSONObject jsonObject = (JSONObject)JSONValue.parse(sJSON);
			jsonObject = (JSONObject)jsonObject.get("DiFi");
			if (!((String)jsonObject.get("status")).equals("SUCCESS")) {
				return null;
			}
			
			jsonObject = (JSONObject)((JSONObject)((JSONArray)((JSONObject)jsonObject.get("response")).get("calls")).get(0)).get("response");
			if (!((String)jsonObject.get("status")).equals("SUCCESS")) {
				return null;
			}
			
			jsonObject = (JSONObject)jsonObject.get("content");
			nextPage = (Boolean)jsonObject.get("more");
			
			JSONArray imageArray = (JSONArray)jsonObject.get("resources");
			
			for (int i = 0; i < imageArray.size(); ++i) {
				String sImageData = (String)((JSONArray)imageArray.get(i)).get(2);
				// string could be empty
				if (sImageData.trim().isEmpty()) continue;
				
				// in the past, stuff between <!-- ^TTT --> and its end tag was used. seems that now dA doesn't neccesarly set those tags anymore
				// but has other ones - an <a tag filled with all the info we need
				// Example:
				// <a super_img="http://fc03.deviantart.net/fs50/i/2009/321/0/4/Reaching_Out_For_Love_by_darklonleyheart.jpg"
				//    super_w="900"
				//    super_h="563"
				//    super_catid="1487"
				//    href="http://darklonleyheart.deviantart.com/art/Reaching-Out-For-Love-143922244?
				//          q=gallery%3Adarklonleyheart+sort%3Atime&amp;qo=0"
				//    title="Reaching Out For Love by ~darklonleyheart, Nov 17, 2009 in&nbsp;Digital&nbsp;
				//           Art&nbsp;>&nbsp;Photomanipulation&nbsp;>&nbsp;Dark"
				// >
				// note: we can't use the super_img tag, because sometimes the image aviable through the download button
				// is larger that that.
				int iStart = sImageData.indexOf("<a");
				sImageData = sImageData.substring(iStart, sImageData.indexOf("\">", iStart) + 2);
				
				// if there is no title tag, the deviation is text (a poem or so) and not supported by GG, skip it
				if (sImageData.indexOf("title=\"") == -1) continue;
				
				// if there is no href tag, the deviation is "in storage", whatever that means for dA. anyway, we can't get it
				if (sImageData.indexOf("href=\"") == -1) continue;
				
				// get title (everything in title tag in front of "by <symbol><user>
				// (this is a bit ugly, but I hate regex and want to get over with it)
				iStart = sImageData.indexOf("title=\"") + 7;
				String sTitleLine = sImageData.substring(iStart, sImageData.indexOf("\"", iStart + 1));
				// q&d fix for special user symbol ° (&#176;)
				sTitleLine = sTitleLine.replaceAll("&#176;", "°");
				String sFavUser = sImageData.substring(sImageData.indexOf("href=\"http://")+13, sImageData.indexOf(".deviantart.com/"));
				// (?i) = ignore case, \Q - \E = ignore regex stuff
				sTitleLine = sTitleLine.replaceAll(" (?i)\\Q" + sFavUser.toLowerCase() + "\\E", Matcher.quoteReplacement(" %" + sFavUser));
				String sTitle = sTitleLine.substring(0, sTitleLine.indexOf(" by %" + sFavUser)) + " ("+sFavUser+")";
				
				// get date (after username in title)
				iStart = sImageData.indexOf("title=\"") + 7;
				iStart = sTitleLine.lastIndexOf(sFavUser + ",") + sFavUser.length() + 1;
				String sDate = sTitleLine.substring(iStart, sTitleLine.indexOf(" in", iStart)).trim();
				DateFormat df = new SimpleDateFormat("MMM d, yyyy", Locale.US);
				df.setTimeZone(TimeZone.getTimeZone(this.TIMEZONE));
				Date imageDate = df.parse(sDate);
				if (date.after(imageDate)) {
					// image too old, abort
					nextPage = false;
					break;
				}

				iStart = sImageData.indexOf("href=\"") + 6;
				String sImageUrl = sImageData.substring(iStart, sImageData.indexOf("\"", iStart + 1));
				try {
					set.add(getImage(new URL(sImageUrl), imageDate, sTitle, sTargetUser)); // targetuser just for error message
				} catch (DownloadException e) {
					// just continue to next image
					continue;
				}
			}
			
			
		} catch (Exception e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
		
		if (nextPage) {
			TreeSet<GrabberImage> tmpSet = getImagesFromAccount(sUrlFirstPart, iStartIdx + 24, sUrlSecondPart, date, sTargetUser);
			if (tmpSet == null) return null;
			set.addAll(tmpSet);
		}
		return set;
	}

	private GrabberImage getImage(URL url, Date dat, String title, String sTargetUser) throws DownloadException {
		return DeviantArt.getImage(url, dat, title, sTargetUser, this.m_ImageFolder, this.PAGENAME);
	}

}
