package Sites;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Locale;
import java.util.TimeZone;
import java.util.TreeSet;

import klaue.DownloadHelper.Downloader;
import Helpers.Account;
import Helpers.Dictionary;
import Helpers.ErrorLogger;
import Helpers.Grabbable;
import Helpers.GrabberImage;
import Helpers.Logwriter;

public class Fchan implements Grabbable {
	final static String PAGENAME = "FChan";
	final static String TIMEZONE = "America/Chicago";
	final static String BASEURL = "http://fchan.us";
	final static HashMap<String, HashSet<String> > IGNOREMAP = new HashMap<String, HashSet<String> >();
	
	// fchan has some sticky threads that should be ignored because they fuck up the date sorting
	// for lack of a better idea, they're blocked by ID for now
	// TODO: find/make better solution for that, it's clumsy
	static {
		HashSet<String> tempSet = new HashSet<String>();
		tempSet.add("25725");
		IGNOREMAP.put("f", tempSet);
		tempSet = new HashSet<String>();
		tempSet.add("66752");
		IGNOREMAP.put("m", tempSet);
		tempSet = new HashSet<String>();
		tempSet.add("13708");
		IGNOREMAP.put("s", tempSet);
		tempSet = new HashSet<String>();
		tempSet.add("28453");
		IGNOREMAP.put("a", tempSet);
		tempSet = new HashSet<String>();
		tempSet.add("31462");
		IGNOREMAP.put("ah", tempSet);
		tempSet = new HashSet<String>();
		tempSet.add("14448");
		IGNOREMAP.put("c", tempSet);
	}

	String m_ImageFolder = "";
	
	public boolean doLogin(Account acc) {
		// "logging in", i.e. accept the disclaymer
		try {
			Downloader.downloadPage(new URL(Fchan.BASEURL + "/disclaimer.pl?agree=1&url=/"),
					Fchan.BASEURL, null, 20000, 3);
			return true;
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			// ignore
			return true;
		} catch (Exception e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return false;
		}
	}
	
	public TreeSet<GrabberImage> getImages(Account acc, String sSavePath, GrabberImage userNotFoundImg) {
		if (acc == null) return null;
		this.m_ImageFolder = sSavePath;
		
		// getting website
		URL url = null;
		//{"Male", "Female", "Herm", "Straight", "Toon", "Alternative", "Alt(hard)", "Clean", "Artist", "Critique"};
		try {
			if (acc.TargetUser.equals("Male")) {
				url = new URL(Fchan.BASEURL + "/m/");
			} else if (acc.TargetUser.equals("Female")) {
				url = new URL(Fchan.BASEURL + "/f/");
			} else if (acc.TargetUser.equals("Herm")) {
				url = new URL(Fchan.BASEURL + "/h/");
			} else if (acc.TargetUser.equals("Straight")) {
				url = new URL(Fchan.BASEURL + "/s/");
			} else if (acc.TargetUser.equals("Toon")) {
				url = new URL(Fchan.BASEURL + "/toon/");
			} else if (acc.TargetUser.equals("Alternative")) {
				url = new URL(Fchan.BASEURL + "/a/");
			} else if (acc.TargetUser.equals("Alt(hard)")) {
				url = new URL(Fchan.BASEURL + "/ah/");
			} else if (acc.TargetUser.equals("Clean")) {
				url = new URL(Fchan.BASEURL + "/c/");
			} else if (acc.TargetUser.equals("Artist")) {
				url = new URL(Fchan.BASEURL + "/artist/");
			} else if (acc.TargetUser.equals("Critique")) {
				url = new URL(Fchan.BASEURL + "/crit/");
			} else {
				TreeSet<GrabberImage> mReturnSet = new TreeSet<GrabberImage>();
				mReturnSet = new TreeSet<GrabberImage>();
				mReturnSet.add(userNotFoundImg);
				return mReturnSet;
			}
		} catch (MalformedURLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
		
		// wenn null retourkommt dann ist es ein fehler, wenn das set leer kommt
		// gibt's keine neuen bilder
		return getImagesFromAccount(url, acc.getDate(), acc.TargetUser);
	}

	public boolean isFromThisSite(Account acc) {
		if (acc.PageName.equals(Fchan.PAGENAME)) return true;
		return false;
	}

	public boolean needsUsernameForAllContent() {
		return false;
	}

	public boolean needsPasswordForAllContent() {
		return false;
	}

	public boolean fixedTargetUsers() {
		return true;
	}

	public String getTargetUserDescription() {
		return Dictionary.getInstance().getValue("SitesUserDescrFchan");
	}

	public String[] getFixedTargetUsers() {
		return new String[] {"Male", "Female", "Herm", "Straight", "Toon", "Alternative", "Alt(hard)", "Clean", "Artist", "Critique"};
	}

	public String getSitename() {
		return Fchan.PAGENAME;
	}

	public String getDescription() {
		return Dictionary.getInstance().getValue("SitesDescrFchan");
	}

	public boolean needsUsernameForSpecialContent() {
		return false;
	}

	public boolean needsPasswordForSpecialContent() {
		return false;
	}
	
	/**
	 * Gets all images from an account, which is in this case male, herm, female etc
	 * @param url the url to the right subpage
	 * @param accountDate the last download date
	 * @param sTargetUser the name of the account (for error messages)
	 * @return a list of images
	 */
	public TreeSet<GrabberImage> getImagesFromAccount(URL url, Date accountDate, String sTargetUser) {
		TreeSet<GrabberImage> set = new TreeSet<GrabberImage>();
		try {
			ArrayList<String> page = Downloader.downloadPage(url, null, null, 20000, 3);
			
			// read the threadlist at the top of the page
			ArrayList<String> threadUrls = new ArrayList<String>();
			boolean inThreadList = false;
			for (String line : page) {
				if (!line.trim().equals("<div id=\"subjectlist\">") && !inThreadList) continue;
				inThreadList = true;
				
				if (line.contains("</div>")) {
					break; // we don't need anything after the threadlist
				}
				
				// filter all the threads out that don't link to an anchor (beginning with #)
				// currently, the thread list is one per line, but fchan likes to change this for no reasons
				StringBuffer bufLine = new StringBuffer(line);
				while (true) {
					int idxStart = bufLine.indexOf("<a href=\"/");
					if (idxStart == -1) break; // not while-argument for performance reasons
					int linkStart = idxStart + 9;
					threadUrls.add(bufLine.substring(linkStart, bufLine.indexOf("\"", linkStart)));
					bufLine.delete(idxStart, linkStart); // remove <a href="
				}
			}
			
			// it would be faster if I first searched through the threads posted on the front page before going through the threadlist, as
			// 10 threads are allready loaded there, but fuck it, if they contain new images, I'd have to DL the whole thread anyway so the
			// speed improvement would've been marginal. Also, it's way easier that way
			for (String threadUrl : threadUrls) {
				// get "area-code" and thread id for checking of stickies
				String area = threadUrl.substring(1, threadUrl.indexOf('/', 1));
				String threadId = threadUrl.substring(threadUrl.lastIndexOf('/') + 1, threadUrl.indexOf(".html"));
				if (IGNOREMAP.containsKey(area) && IGNOREMAP.get(area).contains(threadId)) continue;
				
				threadUrl = Fchan.BASEURL + threadUrl; // /h/res/590.html -> http://fchan.us/h/res/590.html
				TreeSet<GrabberImage> imagesFromThread = getImagesFromThread(new URL(threadUrl), accountDate, sTargetUser);
				// getImagesFromThread returns null if the thread contains not a single post since accountDate
				// through the ordering of the threads, that means we can abort
				if (imagesFromThread == null) break;
				
				set.addAll(imagesFromThread);
			}
			
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		} catch (Exception e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
		return set;
	}
	
	/**
	 * Downloads all the images of the thread since accountDate
	 * @param url the URL of the thread
	 * @param accountDate the date of the last download or to download from
	 * @param sTargetUser the subpage (herm etc) for error messages
	 * @return the images in this thread since accountDate (may be empty) or null if no post, image or not, in this thread is newer than accountDate
	 */
	public TreeSet<GrabberImage> getImagesFromThread(URL url, Date accountDate, String sTargetUser) {
		TreeSet<GrabberImage> set = new TreeSet<GrabberImage>();

		try {
			ArrayList<String> thread = Downloader.downloadPage(url, null, null, 20000, 3);
			
			// filter out the threadcontent
			String title = "";
			boolean inWholeThread = false;
			boolean inPost = false;
			ArrayList<ArrayList<String> > posts = new ArrayList<ArrayList<String> >();
			ArrayList<String> currentPost = new ArrayList<String>();
			for (String line : thread) {
				if (line.trim().isEmpty()) continue;
				
				if (line.contains("<div id=\"whole_thread")) { // <div id="whole_thread590"> for threadid #590
					inWholeThread = true;
					continue;
				}
				if (!inWholeThread) continue;
				
				// grab the title
				if (title.isEmpty() && line.contains("<span class=\"filetitle\">")) {
					int startIdx = line.indexOf("<span class=\"filetitle\">") + 24;
					title = line.substring(startIdx, line.indexOf("</span>", startIdx));
					
					// can contain an image, for example the flash icon, so carve that out
					if (title.contains("<img")) {
						title = title.substring(0, title.indexOf("<img")).trim();
					}
					continue;
				}
				
				// filter out the posts themselfes - they're conveniently wrapped in a table and those tables are the only one in the whole_thread-div
				// luckily, there aren't any more tables after the last post, so no need to check for end of whole_thread-div
				if (line.contains("<table>")) {
					inPost = true;
					continue; // the rest of the line can be omitted
				}
				
				if (inPost) {
					if (line.contains("</table>")) {
						inPost = false;
						posts.add(currentPost);
						currentPost = new ArrayList<String>();
						continue;
					}
					currentPost.add(line);
				}
			}
			
			// traverse posts. start with the last one which is the newest
			for (int i = posts.size() - 1; i >= 0; --i) {
				try {
					GrabberImage img = getImageFromPost(posts.get(i), title, url.toString(), accountDate, sTargetUser);
					if (img == null) continue; // post w/o image or flash
					set.add(img);
				} catch (TooOldException e) {
					if (i == posts.size() -1) {
						// even the newest post itt is older than accountDate
						return null;
					}
					break; // finished thread
				}
			}
			
		}  catch (FileNotFoundException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			ErrorLogger.getInstance().addError(Fchan.PAGENAME, sTargetUser, url.toString(), "FileNotFoundException - " + e.getMessage());
		} catch (Exception e) {
			e.printStackTrace();
			ErrorLogger.getInstance().addError(Fchan.PAGENAME, sTargetUser, url.toString(), e.getMessage());
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
		}
		return set;
	}

	/**
	 * Downloads all the images of the thread since accountDate
	 * @param post the content of the post
	 * @param title the title of the thread it is from
	 * @param threadURL the url of the thread for links
	 * @param accountDate the date of the last download or to download from
	 * @param sTargetUser the subpage (herm etc) for error messages
	 * @return the image in this post or null if it contains no image (or an error happened)
	 * @throws TooOldException if the post date is older than the accoundDate
	 * @throws IOException 
	 */
	public GrabberImage getImageFromPost(ArrayList<String> post, String title, String threadURL, Date accountDate, String sTargetUser) throws TooOldException, IOException {
		Date postDate = null;
		// first and foremost, grab the date - if it's too old, we don't have to do anything else
		// it's a bit tricky since the line has no identifier, it looks like 10/11/28(Sun)14:00</label> (but the label may or may not be on the next line)
		for (int i = 0; i < post.size(); ++i) {
			String line = post.get(i).trim();
			// kill me now..
			String rgx = "^\\d\\d/(0[1-9]|1[012])/(0[1-9]|[12][0-9]|3[01])\\((Mon|Tue|Wed|Thu|Fri|Sat|Sun)\\)([01][0-9]|2[0-3]):[0-5][0-9](\\Q</label>\\E)*$";
			if (line.matches(rgx)) {
				String date = "";
				if (line.endsWith("</label>")) {
					date = line.substring(0, line.indexOf('<'));
				} else {
					// check if </label> is on the next line - if not, continue searching
					if (post.size() != i + 1 && post.get(i+1).trim().endsWith("</label>")) {
						date = line;
					} else {
						continue;
					}
				}
				DateFormat df = new SimpleDateFormat("yy/MM/dd(EEE)HH:mm", Locale.US);
        		df.setTimeZone(TimeZone.getTimeZone(Fchan.TIMEZONE));
        		try {
					postDate = df.parse(date);
				} catch (ParseException e) {
					// cannot happen because we allready validate it with the regex, but oh well, skip
					return null;
				}
				break;
			}
		}
		if (accountDate.after(postDate)) {
			throw new TooOldException();
		}
		
		// get downloadurl, original url, source and description
		String downloadURL = null;
		String originalURL = null;
		String source = null;
		StringBuffer description = new StringBuffer();;
		boolean inDescription = false;
		for (int i = 0; i < post.size(); ++i) {
			String line = post.get(i).trim();
			// downloadurl
			if (downloadURL == null && line.contains("<span class=\"filesize\">")) {
				int startIdx = line.indexOf("href=\"") + 6;
				if (startIdx == 5) {
					// it can happen in some weird cases (i have no idea why) that the link is on the next line
					line = post.get(i + 1).trim();
					startIdx = line.indexOf("href=\"") + 6;
				}
				downloadURL = Fchan.BASEURL + line.substring(startIdx, line.indexOf('"', startIdx));
				continue;
			}
			// orig url
			if (originalURL == null && line.contains("src=\"/fcp/link.gif\"")) {
				int startIdx = line.indexOf("href=\"") + 6;
				originalURL = Fchan.BASEURL + line.substring(startIdx, line.indexOf('"', startIdx));
				continue;
			}
			// source (optional)
			if (source == null && line.contains("<small>Source:")) {
				int startIdx = line.indexOf("<small>");
				source = line.substring(startIdx, line.indexOf("</small>", startIdx) + 8);
				continue;
			}
			// description
			if (!inDescription && line.contains("<blockquote")) {
				inDescription = true;
				continue;
			}
			if (inDescription) {
				line = line.trim();
				if (line.equals("</blockquote>")) {
					inDescription = false;
					//continue; is at the end of the post anyway
					break;
				}
				description.append(line).append('\n');
			}
		}
		if (downloadURL == null) return null; // post w/o file
		if (originalURL == null) originalURL = threadURL; // should never happen
		
		if (source != null) {
			description.insert(0, source + "\n");
		}
		
		
		// now download and save the image
		String localAdress = this.m_ImageFolder + downloadURL.substring(downloadURL.lastIndexOf("/") + 1);
		try {
			localAdress = Downloader.downloadFile(new URL(downloadURL), localAdress, null, null, 20000, 3);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			ErrorLogger.getInstance().addError(Fchan.PAGENAME, sTargetUser, downloadURL, "FileNotFoundException - " + e.getMessage());
			return null;
		} catch (IOException e) {
			if (e.getMessage().indexOf("Server returned HTTP response code") == -1) {
				throw e;
			}
			e.printStackTrace();
			ErrorLogger.getInstance().addError(Fchan.PAGENAME, sTargetUser, downloadURL, e.getMessage());
			return null;
		}
		if (localAdress == null) return null;
		
		return new GrabberImage(localAdress, title, description.toString(), postDate, originalURL);
	}
}

class TooOldException extends Exception {
	// empty
}