package Sites;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.Locale;
import java.util.TimeZone;
import java.util.TreeSet;

import klaue.DownloadHelper.Downloader;
import DownloadHelpers.WrongUsernameOrPasswordException;
import Helpers.Account;
import Helpers.Dictionary;
import Helpers.ErrorLogger;
import Helpers.Grabbable;
import Helpers.GrabberImage;
import Helpers.Logwriter;

public class FurAffinity implements Grabbable {
	final String PAGENAME = "FurAffinity";
	final String LOGIN_PGE = "https://www.furaffinity.net/login/";
	final String BASE_URL = "http://www.furaffinity.net";
	final String USER_CONTROL_PANEL_URL = "http://www.furaffinity.net/controls/settings/";
	final String ARTIST_URL_MAIN = "http://www.furaffinity.net/user/"; // + username/
	final String ARTIST_URL_GALLERY = "http://www.furaffinity.net/gallery/"; // + username/
	final String ARTIST_URL_SCRAPS = "http://www.furaffinity.net/scraps/"; // + username/
	final String TIMEZONE = "America/New_York";
	final String USER_AGENT = "Mozilla/5.0 (Windows NT 5.1; rv:31.0) Gecko/20100101 Firefox/31.0";
	
	GrabberImage m_fourOhFourImg = null;
	String m_ImageFolder = "";

	public boolean doLogin(Account acc) throws WrongUsernameOrPasswordException {
		if (acc.getPassword().equals("") || acc.getUser().equals("")) {
			// user diddnt set an username/pass, so no logging in. poor user will not get any
			// smut *harrharr*
			return true;
		}
		
		String sPost = "action=login&retard_protection=1&name=" + acc.getUser() + "&pass=" + acc.getPassword() + "&login=Login to&nbsp;FurAffinity";
		try {
			// since FA changed the login to SSL (HTTPS), they use a server side "HTTP 302 moved temporarily" to redirect
			// to the main site instead of a normal PHP redirect. This is not handled by DownloadHelper (because it's
			// usally a really special case) and the resulting ArrayList is therefore 0 lines long and an exception is sent, no matter if the login
			// succeeded or not. Therefore, to test if we're logged in, we try to access the user control panel.
			// if that succeeds, we're logged in, if not, the username/password was probably wrong
			try {
				Downloader.sendPOST(new URL(this.LOGIN_PGE), sPost, this.BASE_URL, this.USER_AGENT, 20000, 3);
			} catch (IOException e) {
				String sErrMsg = e.getMessage();
				if (sErrMsg.indexOf("HTTP response code: 302") == -1) {
					// not a redirect, rethrow
					throw e;
				}
			}
			ArrayList<String> test = Downloader.downloadPage(new URL(this.USER_CONTROL_PANEL_URL), this.BASE_URL, this.USER_AGENT, 20000, 3);
			for (String s : test) {
				if (s.indexOf("Please log in!") != -1) {
					throw new WrongUsernameOrPasswordException(acc.getUser(), acc.getPassword());
				}
			}
			return true;
		} catch (MalformedURLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return false;
		} catch (IOException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return false;
		}
	}

	public boolean fixedTargetUsers() {
		return false;
	}

	public String getDescription() {
		return Dictionary.getInstance().getValue("SitesDescrFurAffinity");
	}

	public String[] getFixedTargetUsers() {
		return null;
	}

	public TreeSet<GrabberImage> getImages(Account acc, String sSavePath, GrabberImage userNotFoundImg) {
		if (acc == null) return null;
		this.m_ImageFolder = sSavePath;
		this.m_fourOhFourImg = userNotFoundImg;
		TreeSet<GrabberImage> returnSet = new TreeSet<GrabberImage>();
		
		// checking if user exists
		URL url = null;
		try {
			url = new URL(this.ARTIST_URL_MAIN + acc.TargetUser + "/");
			ArrayList<String> resp = Downloader.downloadPage(url, this.BASE_URL, this.USER_AGENT, 20000, 3);
			for (String s : resp) {
				if (s.indexOf("Fatal system error!") != -1) {
					TreeSet<GrabberImage> mReturnSet = new TreeSet<GrabberImage>();
					mReturnSet.add(userNotFoundImg);
					return mReturnSet;
				}
			}
		} catch (MalformedURLException e) {
			returnSet.add(userNotFoundImg);
			return returnSet;
		} catch (FileNotFoundException e) {
			returnSet.add(userNotFoundImg);
			return returnSet;
		} catch (IOException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
		
		// getting gallery
		try {
			url = new URL(this.ARTIST_URL_GALLERY + acc.TargetUser + "/");
		} catch (MalformedURLException e) {
			// if it were a prob with the username, it would've been exited above, so
			// this has to be something weird
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
		
		TreeSet<GrabberImage> tmpSet = getImagesFromAccount(url, acc.getDate(), acc.TargetUser);
		if (tmpSet == null) return null;
		returnSet.addAll(tmpSet);
		
		// getting scraps
		try {
			url = new URL(this.ARTIST_URL_SCRAPS + acc.TargetUser + "/");
		} catch (MalformedURLException e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
		tmpSet = getImagesFromAccount(url, acc.getDate(), acc.TargetUser);
		if (tmpSet != null) returnSet.addAll(tmpSet);
		return returnSet;
	}

	public String getSitename() {
		return this.PAGENAME;
	}

	public String getTargetUserDescription() {
		return Dictionary.getInstance().getValue("SitesUserDescrFurAffinity");
	}

	public boolean isFromThisSite(Account acc) {
		if (acc.PageName.equals(this.PAGENAME)) return true;
		return false;
	}

	public boolean needsPasswordForAllContent() {
		return false;
	}

	public boolean needsPasswordForSpecialContent() {
		return true;
	}

	public boolean needsUsernameForAllContent() {
		return false;
	}

	public boolean needsUsernameForSpecialContent() {
		return true;
	}

	private TreeSet<GrabberImage> getImagesFromAccount(URL mainUrl, Date date, String sTargetUser) {
		TreeSet<GrabberImage> set = new TreeSet<GrabberImage>();
		boolean bIsValidSite = true; // url for next page, if any
		try {
			ArrayList<String> page = Downloader.downloadPage(mainUrl, this.BASE_URL, this.USER_AGENT, 20000, 3);

			ArrayList<String> previewPics = new ArrayList<String>();
			for (String sLine : page) {
				if (sLine.indexOf("There are no submissions to list") != -1) {
					bIsValidSite = false;
					break;
				}
				
				if (!sLine.contains("class=\"icon\"")) {
					continue;
				}
				
				// there's only one link in the submission part, the one to the view page
				if (sLine.indexOf("<a href=") == -1) continue;
				
				// only images and flash, no text (t-text), no music (t-audio)
				if (sLine.indexOf("t-image") == -1 && sLine.indexOf("t-flash") == -1) continue;
				
				int iStart = sLine.indexOf("<a href=\"") + 9;
				int iEnd = sLine.indexOf("\">", iStart);
				String sUrl = sLine.substring(iStart, iEnd);
				// replace "view" with "full" to get the full sized image
				if (sUrl.indexOf("view") != -1) {
					sUrl = sUrl.replace("view", "full");
				}
				
				previewPics.add(this.BASE_URL + sUrl);
			}
			
			// handle the pics
			for (String sImg : previewPics) {
				GrabberImage img;
				try {
					img = getImage(new URL(sImg), date, sTargetUser);
				} catch (DownloadException e) {
					System.err.println("Continuing after error");
					continue;
				}
				if (img == null) {
					// this and following pics are too old
					bIsValidSite = false;
					break;
				}
				set.add(img);
				
				// give FA time to breathe
				System.out.println("Sleep a sec for FA not to think we're DDOSing");
				Thread.sleep(1000);
			}
			
		} catch (Exception e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			return null;
		}
		
		if (bIsValidSite) {
			// prepare next page
			String sMainUrl = mainUrl.toString();
			if (sMainUrl.charAt(sMainUrl.length() - 1) == '/') {
				sMainUrl = sMainUrl.substring(0, sMainUrl.length() - 1);
			}
			
			String sNextUrl = "";
			try {
				int i = Integer.parseInt(sMainUrl.substring(sMainUrl.lastIndexOf('/') + 1)) + 1;
				sNextUrl = sMainUrl.substring(0, sMainUrl.lastIndexOf('/')) + "/" + i + "/";
			} catch (NumberFormatException e) {
				// first page, without number
				sNextUrl = sMainUrl + "/2/";
			}
			
			URL nextURL;
			try {
				nextURL = new URL(sNextUrl);
			} catch (MalformedURLException e) {
				e.printStackTrace();
				e.printStackTrace(Logwriter.getInstance().getPrintStream());
				return null;
			}
			
			TreeSet<GrabberImage> tmpSet = getImagesFromAccount(nextURL, date, sTargetUser);
			if (tmpSet == null) return null;
			set.addAll(tmpSet);
		}
		return set;
	}
	
	private GrabberImage getImage(URL url, Date date, String sTargetUser) throws DownloadException {
		GrabberImage img = null;
		try {
			String sTitle = "";
			String sDescr = "";
			String sServerImagePath = "";
			String sLocalImgPath = "";
			Date dat = null;
			
			boolean bInComment = false;
			boolean bInTitle = false;
			ArrayList<String> imgPage = Downloader.downloadPage(url, this.BASE_URL, this.USER_AGENT, 20000, 3);
			int specialFaRetries = 3;
			while (imgPage.isEmpty() && specialFaRetries > 0) {
				--specialFaRetries;
				System.out.println("FA returned empty page - waiting 5 seconds and retrying (" + specialFaRetries + "retries remaining)");
				try {Thread.sleep(5000);} catch (Exception e) {}
				imgPage = Downloader.downloadPage(url, this.BASE_URL, this.USER_AGENT, 20000, 3);
			}
			
			if (imgPage.isEmpty()) {
				// still empty, wtf
				System.err.println("Download failed: FA returned empty page even after 3 retries");
				ErrorLogger.getInstance().addError(this.PAGENAME, sTargetUser, url.toString(), "Download failed: FA returned empty page even after 3 retries");
				throw new DownloadException();
			}
			
			for (String sLine : imgPage) {
				if (sLine.trim().equals("")) continue;
				
				// title
				if (sLine.indexOf("<td valign=\"top\" align=\"left\" class=\"cat\">") != -1) {
					bInTitle = true;
					continue;
				}
				if (bInTitle) {
					sTitle = sLine.substring(sLine.indexOf("<b>") + 3, sLine.indexOf("</b>"));
					bInTitle = false;
					continue;
				}
				
				// image (if not flash)
				if (sLine.indexOf("<img id=\"submissionImg\"") != -1) {
					int iStart = sLine.indexOf("src=\"") + 5;
					int iEnd = sLine.indexOf("\"", iStart);
					sServerImagePath = sLine.substring(iStart, iEnd).trim();
					sLocalImgPath = this.m_ImageFolder + sServerImagePath.substring(sServerImagePath.lastIndexOf("/") + 1);
					//sLocalImgPath = Downloader.downloadImage(new URL(sImgPath), sLocalImgPath, null);
					continue;
				}
				
				// flash (if not image)
				if (sLine.indexOf("<param name=\"movie\"") != -1) {
					int iStart = sLine.indexOf("value=\"") + 7;
					int iEnd = sLine.indexOf("\"", iStart);
					sServerImagePath = sLine.substring(iStart, iEnd).trim();
					sLocalImgPath = this.m_ImageFolder + sServerImagePath.substring(sServerImagePath.lastIndexOf("/") + 1);
					//sLocalImgPath = Downloader.downloadImage(new URL(sImgPath), sLocalImgPath, null);
					continue;
				}
				
				// date
				if (sLine.indexOf("&nbsp;&nbsp;&nbsp;&nbsp; <b>Posted:</b>") != -1) {
					// depending on the settings of the user, either the title or the normal
					// text has a usable date
					int iStart = sLine.indexOf("<span title=\"") + 13;
					int iEnd = sLine.indexOf("\"", iStart);
					String sDateTitle = sLine.substring(iStart, iEnd);
					iStart = sLine.indexOf("\">", iEnd) + 2;
					iEnd = sLine.indexOf("</span>", iStart);
					String sDateText = sLine.substring(iStart, iEnd);
					
					// now we need to know which one's the usable one
					// that's easy, the wrong one has always an "ago" in it
					String sDate = (sDateTitle.indexOf("ago") == -1) ? sDateTitle : sDateText;
					
	        		dat = stringToDate(sDate);
					if (date.after(dat)) return null;
				}
				
				// description
				int commentStart = sLine.indexOf("<td valign=\"top\" align=\"left\" width=\"70%\" class=\"alt1\">");
				if (commentStart != -1) {
					bInComment = true;
					sLine = sLine.substring(commentStart + 55);
					if (sLine.trim().isEmpty()) continue;
				}				
				if (bInComment && sLine.indexOf("</td>") != -1) {
					//end of comment. we don't need anything after this
					sDescr += sLine.substring(0, sLine.indexOf("</td>")).trim();
					
					// repair links
					sDescr = sDescr.replace("<a href=\"/user/", "<a target=\"_blank\" href=\"http://www.furaffinity.net/user/");
					sDescr = sDescr.replace("<img src=\"//", "<img src=\"http://");
					//sDescr.indexOf("<a href=\"/user/");
					break;
				}
				if (bInComment) {
					sDescr += sLine.trim();
				}
			}
			if (sServerImagePath.equals("")) {
				for (String s : imgPage) {
					System.out.println("##PAGEDUMP ]" + s);
				}
			}
			
			// for some reason, FA puts in the img links without the http in front of it "//d.facdn.net/art/[...]"
			if (!sServerImagePath.startsWith("http")) {
				sServerImagePath = "http:" + sServerImagePath;
			}
			sLocalImgPath = Downloader.downloadFile(new URL(sServerImagePath), sLocalImgPath, url.toString(), this.USER_AGENT, 20000, 3);
			img = new GrabberImage(sLocalImgPath, sTitle, sDescr, dat, url.toString());
		} catch (Exception e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			String sMessage = e.getLocalizedMessage();
			if (sMessage == null) {
				sMessage = e.toString();
			} else {
				// remove url from message
				if (sMessage.lastIndexOf("http://") != -1) sMessage = sMessage.substring(0, sMessage.lastIndexOf("http://"));
				// remove everything after :
				if (sMessage.lastIndexOf(":") != -1) sMessage = sMessage.substring(0, sMessage.lastIndexOf(":"));
				String sClassdef = e.getClass().toString();
				sMessage = sClassdef.substring(sClassdef.indexOf("class ") + 6) + ((sMessage.length() == 0) ? "" : ": " + sMessage);
			}
			
			ErrorLogger.getInstance().addError(this.PAGENAME, sTargetUser, url.toString(), "Download failed: " + sMessage);
			throw new DownloadException();
		}
		return img;
	}
	
	private Date stringToDate(String sDate) throws ParseException {
		String sFormatString = "";
		if (sDate.contains("st,")) {
			sFormatString = "MMM d'st', yyyy h:mm a";
		} else if (sDate.contains("nd,")) {
			sFormatString = "MMM d'nd', yyyy h:mm a";
		} else if (sDate.contains("rd,")) {
			sFormatString = "MMM d'rd', yyyy h:mm a";
		} else { //if (sDate.contains("th,")) {
			sFormatString = "MMM d'th', yyyy h:mm a";
		}
		DateFormat df = new SimpleDateFormat(sFormatString, Locale.US);
		df.setTimeZone(TimeZone.getTimeZone(this.TIMEZONE));
		return df.parse(sDate);
	}
	
}
