package Sites;

import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.TimeZone;
import java.util.TreeSet;

import klaue.DownloadHelper.Downloader;
import Helpers.Account;
import Helpers.Dictionary;
import Helpers.ErrorLogger;
import Helpers.Grabbable;
import Helpers.GrabberImage;
import Helpers.Logwriter;

public class VCL implements Grabbable {
	final String PAGENAME = "VCL";
	final String ARTIST_URL = "http://us.vclart.net/vcl/Artists/";
	final String TIMEZONE = "America/New_York";
	
	GrabberImage m_fourOhFourImg = null;
	String m_ImageFolder = "";
	
	public TreeSet<GrabberImage> getImages(Account acc, String sSavePath, GrabberImage userNotFoundImg) {
		if (acc == null) return null;
		this.m_ImageFolder = sSavePath;
		this.m_fourOhFourImg = userNotFoundImg;
		
		// getting website
		URL url = null;
		try {
			url = new URL(this.ARTIST_URL + acc.TargetUser + "/");
		} catch (MalformedURLException e) {
			TreeSet<GrabberImage> mReturnSet = new TreeSet<GrabberImage>();
			mReturnSet.add(userNotFoundImg);
			return mReturnSet;
		}
		
		// if null is returned there's an error, if the set returns empty
		// there are no new images
		try {
			return handleDir(url, acc.getDate(), acc.TargetUser);
		} catch (DownloadException e) {
			return null;
		}
	}

	public boolean doLogin(Account acc) {
		return true;
	}
	
	public boolean isFromThisSite(Account acc) {
		if (acc.PageName.equals(this.PAGENAME)) return true;
		return false;
	}

	public boolean needsUsernameForAllContent() {
		return false;
	}

	public boolean needsPasswordForAllContent() {
		return false;
	}

	public boolean needsUsernameForSpecialContent() {
		return false;
	}

	public boolean needsPasswordForSpecialContent() {
		return false;
	}

	public boolean fixedTargetUsers() {
		return false;
	}

	public String[] getFixedTargetUsers() {
		return null;
	}

	public String getTargetUserDescription() {
		return Dictionary.getInstance().getValue("SitesUserDescrVCL");
	}

	public String getSitename() {
		return this.PAGENAME;
	}

	public String getDescription() {
		return Dictionary.getInstance().getValue("SitesDescrVCL");
	}
	
	public TreeSet<GrabberImage> handleDir(URL url, Date AccountDate, String Username) throws DownloadException {
		TreeSet<GrabberImage> set = new TreeSet<GrabberImage>();
		try {
			ArrayList<String> PageList = Downloader.downloadPage(url, null, null, 20000, 3);
			
			ArrayList<URL> Directories = new ArrayList<URL>();
			ArrayList<URL> ImagePages = new ArrayList<URL>();
			boolean bInDirectories = false;
			boolean bInImages = false;
			for (String sLine : PageList) {
				if (sLine.trim().equals("")) continue;
				
				// just for the first one
				if (sLine.indexOf("<title>VCL -  File Not Found</title>") != -1) {
					set.add(this.m_fourOhFourImg);
					return set;
				}
				
				if (sLine.indexOf("<font color=white>Directories") != -1) {
					bInDirectories = true;
					continue;
				}
				if (bInDirectories && sLine.indexOf("</table>") != -1) {
					bInDirectories = false;
					continue;
				}
				
				if (bInDirectories) {
					// on every line in the directory-part of the page is another
					// directory, no need to check this
					int iStart = sLine.indexOf("<a href=\"") + 9;
					int iEnd = sLine.indexOf("\">", iStart);
					
					//get the imgs wioth the amazing powers of the recursivles!
					// new URL(url.toString() + sLine.substring(iStart, iEnd)) is something like
					// http://us.vclart.net/vcl/Artists/Saffire-Raynius/ + OtherJunk/
					Directories.add(new URL(url.toString() + sLine.substring(iStart, iEnd)));
				}
				
				if (sLine.indexOf("<font color=white>Images - 30 per page") != -1) {
					bInImages = true;
					continue;
				}
				if (bInImages && sLine.indexOf("</table>") != -1) {
					bInImages = false;
					break;
				}
				
				if (bInImages) {
					if (sLine.indexOf("-by-date.html") == -1) continue;
					
					int iStart = sLine.indexOf("</a><td>") + 8;
					int iEnd = sLine.indexOf(" ...", iStart);
					
					if (iEnd == -1) {
						// this can only happen when there's only one pic in the dir..
						iEnd = sLine.indexOf("<td>", iStart);
					}
					
					// date of the newest pic on this line
					String sDate = sLine.substring(iStart, iEnd);
	        		DateFormat df = new SimpleDateFormat("yyyy.MM.dd");
	        		df.setTimeZone(TimeZone.getTimeZone(this.TIMEZONE));
	        		Date dat = df.parse(sDate);
					if (AccountDate.after(dat)) break;
					
					// get the addy
					iStart = sLine.indexOf("<a href=\"") + 9;
					iEnd = sLine.indexOf("\">");
					ImagePages.add(new URL(url.toString() + sLine.substring(iStart, iEnd)));
				}
			}
			
			// get the images out of the imagePages
			for (URL img : ImagePages) {
				TreeSet<GrabberImage> tmpSet;
				try {
					tmpSet = handleImagePage(img, AccountDate, Username);
				} catch (DownloadException e) {
					continue;
				}
				if (tmpSet == null) return null;
				set.addAll(tmpSet);
			}			
			
			// get all the images out of the directories
			for (URL dir : Directories) {
				// call the superpowers of the recursivles!
				TreeSet<GrabberImage> tmpSet;
				try {
					tmpSet = handleDir(dir, AccountDate, Username);
				} catch (DownloadException e) {
					continue;
				}
				if (tmpSet == null) return null;
				set.addAll(tmpSet);
			}			
		} catch (Exception e) {
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			String sMessage = e.getLocalizedMessage();
			if (sMessage == null) {
				sMessage = e.toString();
			} else {
				// remove url from message
				if (sMessage.lastIndexOf("http://") != -1) sMessage = sMessage.substring(0, sMessage.lastIndexOf("http://"));
				// remove everything after :
				if (sMessage.lastIndexOf(":") != -1) sMessage = sMessage.substring(0, sMessage.lastIndexOf(":"));
				String sClassdef = e.getClass().toString();
				sMessage = sClassdef.substring(sClassdef.indexOf("class ") + 6) + ((sMessage.length() == 0) ? "" : ": " + sMessage);
			}
			
			ErrorLogger.getInstance().addError(this.PAGENAME, Username, url.toString(), "Download failed: " + sMessage);
			throw new DownloadException();
		}
		return set;
	}
	
	private TreeSet<GrabberImage> handleImagePage(URL url, Date AccountDate, String Username) throws DownloadException {
		TreeSet<GrabberImage> set = new TreeSet<GrabberImage>();
		URL imgUrl = null;
		try {
			ArrayList<String> PageList = Downloader.downloadPage(url, null, null, 20000, 3);
			
			boolean bImage = false;
			boolean bDescription = false;
			String sAdressOfLocalCopy = "";
			String sDescription = "";
			String sTitle = "";
			Date dateOfImg = Calendar.getInstance().getTime();
			for (String sLine : PageList) {
				if (sLine.trim().equals("")) continue;
				
				if (sLine.indexOf("<tr bgcolor=\"#F6F0EC\">") != -1 || sLine.indexOf("<tr bgcolor=\"#ECE0D8\">") != -1) {
					bImage = true;
					continue;
				}
				if (bImage && sLine.indexOf("</tr>") != -1) {
					bImage = false;
					sAdressOfLocalCopy = Downloader.downloadFile(imgUrl, sAdressOfLocalCopy, null, null, 20000, 3);
					
					// because the description ends with the image, we gotta subtract an eventual
					// <p> at the end here
					sDescription = sDescription.trim();
					if (sDescription.substring(sDescription.length() - 3).equals("<p>")) {
						sDescription = sDescription.substring(0, sDescription.length() - 3).trim();
					}
					
					set.add(new GrabberImage(sAdressOfLocalCopy, sTitle, sDescription, dateOfImg, url.toString()));
					
					// resetting values
					imgUrl = null;
					sAdressOfLocalCopy = "";
					sDescription = "";
					sTitle = "";
					dateOfImg = Calendar.getInstance().getTime();
					bDescription = false;
					continue;
				}
				if (!bImage) continue;
				
				// image
				if (sLine.indexOf("<td valign=\"top\">Image:&nbsp;<a href=\"") != -1) {
					int iStart = sLine.indexOf("<a href=\"") + 9;
					int iEnd = sLine.indexOf("\">", iStart);
					String sFilePath = sLine.substring(iStart, iEnd);
					
					// saving
					sAdressOfLocalCopy = this.m_ImageFolder + Username + "_" + 
						sFilePath.substring(sFilePath.lastIndexOf("/") + 1);
					imgUrl = new URL(sFilePath);
					
					// and now, on the same line, the title, i.e. filename without extension
					iStart = iEnd + 2;
					iEnd = sLine.lastIndexOf(".");
					sTitle = sLine.substring(iStart, iEnd);
				}
				
				// date
				if (sLine.indexOf("&nbsp;bytes&nbsp;") != -1) {
					int iStart = sLine.indexOf("bytes&nbsp;") + 11;
					String sDate = sLine.substring(iStart);
	        		DateFormat df = new SimpleDateFormat("yyyy.MM.dd");
	        		df.setTimeZone(TimeZone.getTimeZone(this.TIMEZONE));
	        		Date dat = df.parse(sDate);
					if (AccountDate.after(dat)) break;
					dateOfImg = dat;
				}
				
				// description
				if (!bDescription && sLine.indexOf("<p>") != -1) {
					bDescription = true;
					sLine = sLine.substring(sLine.indexOf("<p>") + 3);
				}
				if (bDescription && sLine.indexOf("<table") != -1) {
					// the description will usually end with the image and is therefore
					// handled at the same place. this here is just for when some
					// tags were set
					bDescription = false;
				}
				
				if (bDescription) {
					sDescription += sLine;
				}
			}
		} catch (Exception e) {
			String sUrl = url.toString();
			if (imgUrl != null) sUrl = imgUrl.toString();
			
			e.printStackTrace();
			e.printStackTrace(Logwriter.getInstance().getPrintStream());
			String sMessage = e.getLocalizedMessage();
			if (sMessage == null) {
				sMessage = e.toString();
			} else {
				// remove url from message
				if (sMessage.lastIndexOf("http://") != -1) sMessage = sMessage.substring(0, sMessage.lastIndexOf("http://"));
				// remove everything after :
				if (sMessage.lastIndexOf(":") != -1) sMessage = sMessage.substring(0, sMessage.lastIndexOf(":"));
				String sClassdef = e.getClass().toString();
				sMessage = sClassdef.substring(sClassdef.indexOf("class ") + 6) + ((sMessage.length() == 0) ? "" : ": " + sMessage);
			}
			
			ErrorLogger.getInstance().addError(this.PAGENAME, Username, sUrl, "Download failed: " + sMessage);
			throw new DownloadException();
		}
		return set;
	}

}
